/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _ELEMENT_BROWSER_H_
#define _ELEMENT_BROWSER_H_

#include <KCategorizedView>

class KCategorizedView;
class ElementModel;
class KCategorizedSortFilterProxyModel;
class KSycoca;
class QTimer;

class ElementBrowser : public KCategorizedView
{
    Q_OBJECT

public:
    ElementBrowser( QWidget* parent = 0 );
    ~ElementBrowser();

public Q_SLOTS:
    /**
     * Update the plugin (fire a timer) list when notified by KSycoca
     */
    void triggerPluginUpdate(QStringList resources);
    
    /**
     * Actual. update function
     */
    void refresh();

    /**
     * Black box operations: open, delete and export to filesystem
     */
    void slotOpenBlackBox();
    void slotDeleteBlackBox();
    void slotExportBlackBox();
        
private Q_SLOTS:
    void init();
    void setupActions();

protected:
    /**
     * Work around KCategorizedView weirdness
     */
    void startDrag( Qt::DropActions supportedActions );
    
    void contextMenuEvent ( QContextMenuEvent * event );

private:
    ElementModel* m_model;
    KCategorizedSortFilterProxyModel* m_proxyModel;
    KSycoca *m_sycoca;
                
    class Private;
    Private* const d;
};

#endif
