/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "konduitscene.h"

#include <QtGui/QGraphicsSceneDragDropEvent>
#include <QtCore/QMimeData>
#include <QtCore/QDataStream>
#include <QtCore/QTimer>

//#include <plasma/phase.h>
#include <plasma/animator.h>
#include <plasma/applet.h>
#include <plasma/containment.h>

#include <kdebug.h>

#include "../src/element.h"
#include "../src/blackboxloaderelement.h"


KonduitScene::KonduitScene( QObject* parent )
    : Corona( parent ),
      m_desktop( 0 )
{
    //QTimer::singleShot(0, this, SLOT( init() ));
    init();
}


KonduitScene::~KonduitScene()
{

    //Any QGraphicsItem on the scene is deleted by ~QGraphicScene.
    /*
    foreach(QGraphicsItem* i, items()) {
        removeItem(i);//otherwise Konduit crashes on exit
        //((Plasma::Applet*) i)->destroy();
    }
    */
}


void KonduitScene::init()
{
    //setFocusPolicy( Qt::ClickFocus );
    //setFocus();
    m_desktop = addContainment( "null" );
    m_desktop->init();
    m_desktop->setFormFactor(Plasma::Planar);
    //m_desktop->flushUpdatedConstraints();

    connect( this, SIGNAL( sceneRectChanged( const QRectF& ) ),
             this, SLOT( slotSceneRectChanged( const QRectF& ) ) );
}

void KonduitScene::slotSceneRectChanged( const QRectF& rect )
{
//    m_desktop->setGeometry( rect );
}


void KonduitScene::dragEnterEvent( QGraphicsSceneDragDropEvent* event )
{
    event->setAccepted( event->mimeData()->hasFormat("text/x-konduitelementname" ) );
}


void KonduitScene::dragMoveEvent( QGraphicsSceneDragDropEvent* event )
{
    Q_UNUSED( event );
    // do nothing
}


void KonduitScene::dropEvent( QGraphicsSceneDragDropEvent* event )
{
    if( event->mimeData()->hasFormat( "text/x-konduitelementname" ) ) {
        QByteArray data = event->mimeData()->data( "text/x-konduitelementname" );
        QStringList names;
        QDataStream s( data );
        s >> names;

        foreach( QString name, names ) {
            createElement( name, event->scenePos() );
        }

        event->accept();
    }
}


void KonduitScene::createElement( const QString& name, const QPointF& pos )
{
    QVariantList args;
    int id = 0;
    
    Plasma::Applet* applet = Plasma::Applet::load( name, id, args );;
    
    Konduit::Elements::BlackboxLoaderElement* bb = dynamic_cast<Konduit::Elements::BlackboxLoaderElement*>(applet);
    if(bb && bb->failedToLoad()) {
        kDebug() << "Error: cannot load applet" << name;
        return; 
    }

    if ( applet ) {
        addElement( qobject_cast<Konduit::Element*>( applet ), pos );
    }
    else {
        kDebug() << "Failed to create element" << name;
    }
}


void KonduitScene::addElement( Konduit::Element* element, const QPointF& pos, bool dontInit )
{
    m_desktop->addApplet( element, pos, dontInit );
    
    if ( pos != QPointF( -1, -1 ) ) {
        //element->setGeometry( QRectF( pos, element->preferredSize() ) );//plugins resize themselves
    }
    Plasma::Animator::self()->animateItem( element, Plasma::Animator::AppearAnimation );
    //connect(element, SIGNAL(blackBoxSaved( QString )), parent(), SLOT(watchBlackBox( QString )));
    connect(element, SIGNAL(openBlackBox( QString )), 
            parent()->parent(), SLOT(slotOpenBlackBox( QString )));
}

#include "konduitscene.moc"
