/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2008 Sebastian Trueg <trueg@kde.org>
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "konduitselectiondialog.h"
#include "../src/konduitstoragemodel.h"
#include "../src/element.h"

#include <QtGui/QVBoxLayout>
#include <QtGui/QLabel>
#include <QtGui/QListWidget>

#include <KLocale>


class KonduitSelectionDialog::Private
{
public:
    Private( KonduitSelectionDialog* parent )
        : q( parent ) {
    }

    QListWidget* konduitList;

    void _k_slotSelectionChanged();

private:
    KonduitSelectionDialog* q;
};


void KonduitSelectionDialog::Private::_k_slotSelectionChanged()
{
    q->enableButtonOk( !konduitList->selectedItems().isEmpty() );
}


KonduitSelectionDialog::KonduitSelectionDialog( QWidget* parent, bool isBlackBox )
    : KDialog( parent ),
      d( new Private( this ) )
{
    setButtons( Ok|Cancel );
    QWidget* w = mainWidget();
    QVBoxLayout* lay = new QVBoxLayout( w );
    lay->setMargin( 0 );
    lay->setSpacing( spacingHint() );
    lay->addWidget( new QLabel( i18n( "Please select a Konduit from the list:" ), w ) );
    d->konduitList = new QListWidget( w );
    lay->addWidget( d->konduitList );

    Konduit::StorageModel *sm = Konduit::Element::globalStorageModel();
    d->konduitList->addItems( isBlackBox ? sm->allSavedKonduitBlackBoxes() : sm->allSavedKonduits() );

    connect( d->konduitList, SIGNAL( itemSelectionChanged() ),
             this, SLOT( _k_slotSelectionChanged() ) );
    connect( d->konduitList, SIGNAL( itemDoubleClicked( QListWidgetItem* ) ),
             this, SLOT( accept() ) );

    d->_k_slotSelectionChanged();
}


KonduitSelectionDialog::~KonduitSelectionDialog()
{
    delete d;
}


QString KonduitSelectionDialog::selectKonduit( QWidget* parent, bool isBlackBox )
{
    KonduitSelectionDialog dlg( parent, isBlackBox );
    if ( dlg.exec() ) {
        return dlg.d->konduitList->selectedItems().first()->text();
    }
    else {
        return QString();
    }
}

#include "konduitselectiondialog.moc"
