/*
 * konduitshell.h
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 */
#ifndef KONDUITSHELL_H
#define KONDUITSHELL_H


#include <kxmlguiwindow.h>

#include "ui_prefs_base.h"

class KonduitShellView;
class QPrinter;
class KToggleAction;
class KUrl;
class KAction;
namespace Konduit {
    class StorageModel;
}

/**
 * This class serves as the main window for KonduitShell.  It handles the
 * menus, toolbars, and status bars.
 *
 * @short Main window class
 * @author Sebastian Trueg <trueg@kde.org>
 * @version 0.1
 */
class KonduitShell : public KXmlGuiWindow
{
    Q_OBJECT

public:
    /**
     * Default Constructor
     */
    KonduitShell();

    /**
     * Default Destructor
     */
    virtual ~KonduitShell();

public slots: 
    /**
     * Opens the black box with the given name
     */
    void slotOpenBlackBox( QString name );
    
    /**
     * Deletes the black box with the given name (from the filesystem and the RDF repo)
     */
    void slotDeleteBlackBox( QString name );
    
    /**
    * Export the black box to a given file
    */
    void slotExportBlackBox( QString name );
    
    /**
    * Import the black box to a given file
    */
    void slotImportBlackBox();

private slots:
    void openNew();//deletes the view, and creates a new one
    void fileOpen();
    void fileInsert();
    void fileOpenBlackBox();
    void optionsPreferences();

private:
    void setupActions();

private:
    Ui::prefs_base ui_prefs_base ;
    KonduitShellView *m_view;

    QPrinter   *m_printer;
    KToggleAction *m_toolbarAction;
    KToggleAction *m_statusbarAction;
    Konduit::StorageModel *m_ksm;
    //KAction *m_openBBAction;
};

#endif // _KONDUITSHELL_H_
