#!/usr/bin/env ruby
#
# foaf2gallery.rb

require 'rdf/redland'

# some variables which could be given to the script as parameters
picturesPerRow = 3
outputFileName = "gallery.html"
border = 1
padding = 10
spacing = 0

# I assume the ntriples just arrive on a string:
#ntripleData = ARGV[0]

# or on a file:
# fileName = ARGV[0]
# ntripleData = ""
# File.open(fileName, "r") { |f|
#     ntripleData = f.read
# }

# or on stdin:
ntripleData = STDIN.read

puts "ntripleData: #{ntripleData}"

# we can do this in memory:
storage = Redland::MemoryStore.new
unless storage then
  raise "Failed to create RDF storage"
end

# a model
model = Redland::Model.new(storage)
unless model then
  raise "Failed to create RDF model"
end

# we are expecting ntriples
parser = Redland::Parser.ntriples
unless parser then
  raise "Failed to create RDF parser"
end

parser.parse_string_into_model(model, ntripleData, "http://www.dummybaseuri.org#", nil)

queryString = " PREFIX foaf: <http://xmlns.com/foaf/0.1/> " +
  " SELECT ?name ?picture " +
  " WHERE { ?person foaf:name ?name . " +
  " ?person foaf:depiction ?picture } "

query = Redland::Query.new(queryString)
results=query.execute(model)

# output some HTML
outputFile = File.open(outputFileName, "w")
outputFile.puts "<html>\n"
outputFile.puts "<head><title>Nice FOAF pictures - Look at us nerds!</title></head>\n"
outputFile.puts "<body>\n"
outputFile.puts "<table border='#{border} cellspacing='#{spacing}' cellpadding='#{padding}'>\n"
outputFile.puts "<tr>\n"
count = 0
columnWidth = 100 / picturesPerRow
while !results.finished?()
  if count != 0 && count%picturesPerRow == 0 then
    outputFile.puts "</tr>"
    outputFile.puts "<tr>"
  end
  picture = results.binding_value_by_name("picture").uri
  name = results.binding_value_by_name("name")
  outputFile.puts "<td width='#{columnWidth}%' align='center'>"
  outputFile.puts "<img src='#{picture}'><br/>"
  outputFile.puts "#{name}"
  outputFile.puts "</td>"
  count = count + 1
  results.next()
end
outputFile.puts "</tr>\n"
outputFile.puts "</table>\n"
outputFile.puts "</body></html>"

# I don't really know what useful RDF output I could give... Maybe just a link to the file we created and some annotation?
fullPath = File.expand_path(outputFileName)
puts "<file:#{fullPath}> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://xmlns.com/foaf/0.1/Document> ."
puts "<file:#{fullPath}> <http://www.w3.org/2000/01/rdf-schema#label> \"FOAF Gallery\" ."
puts "<file:#{fullPath}> <http://www.w3.org/2000/01/rdf-schema#comment> \"An HTML page depicting some FOAF people.\" ."

