/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "abstractparseelement.h"

#include <kio/netaccess.h>

#include <Soprano/Parser>
#include <Soprano/Global>
#include <Soprano/RdfSerialization>
#include <Soprano/PluginManager>
#include <Soprano/StatementIterator>


class Konduit::Elements::AbstractParseElement::Private
{
public:
};


Konduit::Elements::AbstractParseElement::AbstractParseElement( QObject* parent, const QVariantList& args )
    : SourceElement( parent, args ),
      d( 0 )
{
    addSocket( "output" );

    ConfigurationOption o( "baseUri", QVariant::Url, QUrl( "http://www.dummybaseuri.org#" ) );
    o.setLabel( i18n( "Base URI" ) );
    addConfigurationOption( o );

    ConfigurationOption serializationOption( "serialization", QVariant::String, Soprano::serializationMimeType( Soprano::SerializationNQuads ) );
    serializationOption.setForceSuggestedValue( true );
    QVariantList serializations;
    serializations << Soprano::serializationMimeType( Soprano::SerializationRdfXml )
                   << Soprano::serializationMimeType( Soprano::SerializationTurtle )
                   << Soprano::serializationMimeType( Soprano::SerializationTrig )
                   << "application/n-triples"
                   << Soprano::serializationMimeType( Soprano::SerializationNQuads );
    serializationOption.setSuggestedValues( serializations );
    serializationOption.setLabel( i18n( "Serialization" ) );
    addConfigurationOption( serializationOption );
}


Konduit::Elements::AbstractParseElement::~AbstractParseElement()
{
}


void Konduit::Elements::AbstractParseElement::setSerialization( Soprano::RdfSerialization s, const QString& userSerialization )
{
    setOption( "serialization", ConfigurationSetting( Soprano::serializationMimeType( s, userSerialization ) ) );
}


Soprano::RdfSerialization Konduit::Elements::AbstractParseElement::serialization() const
{
    QString s = optionSetting( "serialization" ).value().toString();
    return Soprano::mimeTypeToSerialization( s );
}


QString Konduit::Elements::AbstractParseElement::userSerialization() const
{
    return optionSetting( "serialization" ).value().toString();
}


void Konduit::Elements::AbstractParseElement::setBaseUri( const QUrl& url )
{
    setOption( "baseUri", ConfigurationSetting( url ) );
}


QUrl Konduit::Elements::AbstractParseElement::baseUri() const
{
    return optionSetting( "baseUri" ).value().toUrl();
}


Soprano::StatementIterator Konduit::Elements::AbstractParseElement::data( const Socket* socket ) const
{
    Q_UNUSED( socket );

    const Soprano::Parser* parser = Soprano::PluginManager::instance()->discoverParserForSerialization( serialization(), userSerialization() );
    if ( parser ) {
        return parseData( parser );
    }
    else {
        kDebug() << "Failed to find parser for serialization" << userSerialization();
    }

    kDebug() << "failed to parse";
    return Soprano::StatementIterator();
}

#include "abstractparseelement.moc"
