/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _KONDUIT_ABSTRACT_PARSE_ELEMENT_ELEMENT_H_
#define _KONDUIT_ABSTRACT_PARSE_ELEMENT_ELEMENT_H_

#include "sourceelement.h"
#include "konduit_export.h"

#include <Soprano/RdfSerialization>

#include <kurl.h>

namespace Soprano {
    class Parser;
}


namespace Konduit {
    namespace Elements {
        /**
         * Reads data from a file and provides it as an output
         */
        class KONDUIT_EXPORT AbstractParseElement : public SourceElement
        {
            Q_OBJECT

        public:
            AbstractParseElement( QObject* parent, const QVariantList& args );
            virtual ~AbstractParseElement();

            void setSerialization( Soprano::RdfSerialization s, const QString& userSerialization = QString() );
            Soprano::RdfSerialization serialization() const;
            QString userSerialization() const;

            void setBaseUri( const QUrl& url );
            QUrl baseUri() const;

            Soprano::StatementIterator data( const Socket* socket ) const;

        protected:
            virtual Soprano::StatementIterator parseData( const Soprano::Parser* parser ) const = 0;

        private:
            class Private;
            Private* const d;
        };
    }
}

#endif
