/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _KONDUIT_CONFIGURATION_OPTION_H_
#define _KONDUIT_CONFIGURATION_OPTION_H_

#include "konduit_export.h"

#include <QtCore/QSharedDataPointer>
#include <QtCore/QVariant>
#include <QtCore/QString>


namespace Konduit {
    /**
     * Use type QVariant::Url for file paths
     */
    class KONDUIT_EXPORT ConfigurationOption
    {
    public:
        ConfigurationOption();
        ConfigurationOption( const QString& name, QVariant::Type type, const QVariant& defaultValue = QVariant() );
        ConfigurationOption( const ConfigurationOption& other );
        ~ConfigurationOption();

        ConfigurationOption& operator=( const ConfigurationOption& other );

        void setLabel( const QString& label );

        /**
         * \return The label or the name if no label has been set.
         */
        QString label() const;

        QString name() const;
        QVariant::Type type() const;
        bool isListOption() const;
        QList<QVariant> suggestedValues() const;
        bool forceSuggestedValue() const;

        // FIXME: this should be a ConfigurationSetting
        QVariant defaultValue() const;

        QString toolTip() const;
        QString whatsThis() const;

        void setName( const QString& name );
        void setType( QVariant::Type type );
        void setIsListOption( bool b );
        void setSuggestedValues( const QList<QVariant>& list );
        void addSuggestedValue( const QVariant& value );
        void setForceSuggestedValue( bool b );
        void setDefaultValue( const QVariant& value );

        void setToolTip( const QString& text );
        void setWhatsThis( const QString& text );

    private:
        class Private;
        QSharedDataPointer<Private> d;
    };
}

#endif
