/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "configurationsetting.h"

#include <QtCore/QSharedData>

#include <KDebug>

class Konduit::ConfigurationSetting::Private : public QSharedData
{
public:
    QVariantList values;
};

Konduit::ConfigurationSetting::ConfigurationSetting()
    : d( new Private() )
{
}


Konduit::ConfigurationSetting::ConfigurationSetting( const QVariant& value )
    : d( new Private() )
{
    d->values.append( value );
}


Konduit::ConfigurationSetting::ConfigurationSetting( const QVariantList& values )
    : d( new Private() )
{
    d->values = values;
}


Konduit::ConfigurationSetting::ConfigurationSetting( const ConfigurationSetting& other )
{
    d = other.d;
}


Konduit::ConfigurationSetting::~ConfigurationSetting()
{
}


Konduit::ConfigurationSetting& Konduit::ConfigurationSetting::operator=( const ConfigurationSetting& other )
{
    d = other.d;
    return *this;
}


QVariant Konduit::ConfigurationSetting::value() const
{
    if ( !d->values.isEmpty() ) {
        return d->values.first();
    }
    else {
        return QVariant();
    }
}


QList<QVariant> Konduit::ConfigurationSetting::values() const
{
    return d->values;
}


void Konduit::ConfigurationSetting::setValue( const QVariant& value )
{
    d->values.clear();
    d->values.append( value );
}


void Konduit::ConfigurationSetting::setValues( const QVariantList& list )
{
    kDebug() << list;
    d->values = list;
}


void Konduit::ConfigurationSetting::addValue( const QVariant& value )
{
    d->values.append( value );
}
