/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "cord.h"
#include "plug.h"
#include "socket.h"

#include <QtGui/QPainter>
#include <QtGui/QGraphicsSceneMouseEvent>
#include <QtGui/QGraphicsScene>

#include <KDebug>


Konduit::Cord::Cord( Plug* plug, QGraphicsItem* parent )
    : QObject(),
      QGraphicsPathItem( parent ),
      m_plug( plug ),
      m_movingPlug( false )
{
    setZValue( plug->zValue()+1000 );
    reset();
}


Konduit::Cord::~Cord()
{
}


QRectF Konduit::Cord::plugBoundingRect() const
{
    return QRectF( m_plugPosition.x()-13, m_plugPosition.y()-4, 13, 8 );
}


void Konduit::Cord::reset()
{
    QPointF origin( m_plug->scenePos() );
    origin.ry() += 10;
    origin.rx() += 10;

    setPos( origin );

    if ( m_plug && m_plug->socket() ) {
        m_plugPosition = mapFromScene( m_plug->socket()->scenePos() );
        m_plugPosition.ry() += 10;
    }
    else {
        m_plugPosition = QPointF();
    }

    updatePath();
}


void Konduit::Cord::updatePath()
{
    // FIXME: handle orientations and try not to cross any other item
    QPainterPath path = QPainterPath();

    if ( QPointF() != m_plugPosition ) {
        qreal x = qMin( 0.0, m_plugPosition.x() );
        path.lineTo( x-20.0, 0.0 );
        path.lineTo( x-20.0, m_plugPosition.y() );
        path.lineTo( m_plugPosition.x()-10, m_plugPosition.y() );
    }

    // draw the plug
    if ( m_plug && m_plug->socket() ) {
        path.moveTo( m_plugPosition.x(), m_plugPosition.y()-4 );
        path.lineTo( m_plugPosition.x(), m_plugPosition.y()+4 );
        path.arcTo( m_plugPosition.x()-10, m_plugPosition.y()-4, 10, 8, 90.0, 180.0 );
        path.moveTo( m_plugPosition.x(), m_plugPosition.y()-2 );
        path.lineTo( m_plugPosition.x()+3, m_plugPosition.y()-2 );
        path.moveTo( m_plugPosition.x(), m_plugPosition.y()+2 );
        path.lineTo( m_plugPosition.x()+3, m_plugPosition.y()+2 );
    }
    else {
        path.moveTo( m_plugPosition.x()-10, m_plugPosition.y()-4 );
        path.lineTo( m_plugPosition.x()-10, m_plugPosition.y()+4 );
        path.arcTo( m_plugPosition.x()-10, m_plugPosition.y()-4, 10, 8, 270.0, 180.0 );
        path.moveTo( m_plugPosition.x()-10, m_plugPosition.y()-2 );
        path.lineTo( m_plugPosition.x()-13, m_plugPosition.y()-2 );
        path.moveTo( m_plugPosition.x()-10, m_plugPosition.y()+2 );
        path.lineTo( m_plugPosition.x()-13, m_plugPosition.y()+2 );
    }

    setPath( path );
}


void Konduit::Cord::mousePressEvent( QGraphicsSceneMouseEvent* event )
{
    kDebug();
    if ( plugBoundingRect().contains( event->pos() ) ) {
        m_movingPlug = true;
    }
    else {
        QGraphicsPathItem::mousePressEvent( event );
    }
}


void Konduit::Cord::mouseMoveEvent( QGraphicsSceneMouseEvent* event )
{
    if ( m_movingPlug ) {
        m_plugPosition = event->pos();
        updatePath();
    }
    else {
        QGraphicsPathItem::mouseMoveEvent( event );
    }
}


void Konduit::Cord::mouseReleaseEvent( QGraphicsSceneMouseEvent* event )
{
    if ( m_movingPlug ) {
        m_movingPlug = false;
        QList<QGraphicsItem*> allItems = scene()->items( mapToScene( event->pos() ) );
        Q_FOREACH( QGraphicsItem* item, allItems ) {
            Socket* sock = dynamic_cast<Socket*>( item );
            if ( sock ) {
                m_plug->plugIn( sock );
                reset();
                return;
            }
        }

        // nothing found -> unplug
        m_plug->unplug();
        reset();
    }
}

#include "cord.moc"
