/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _KONDUID_CORD_H_
#define _KONDUID_CORD_H_

#include "konduit.h"

#include <QtCore/QObject>
#include <QtCore/QRectF>
#include <QtCore/QPointF>
#include <QtCore/QPointer>
#include <QtGui/QPainterPath>
#include <QtGui/QGraphicsPathItem>


class QPainter;

namespace Konduit {

    class Plug;

    class Cord : public QObject, public QGraphicsPathItem
    {
        Q_OBJECT

    public:
        Cord( Plug* plug, QGraphicsItem* parent = 0 );
        ~Cord();
    
        void reset();

    protected:
        void mousePressEvent( QGraphicsSceneMouseEvent* event );
        void mouseMoveEvent( QGraphicsSceneMouseEvent* event );
        void mouseReleaseEvent( QGraphicsSceneMouseEvent* event );

    private:
        void updatePath();
        void setPlugPosition( const QPointF& pos, Orientation orientation = ORIENTATION_RIGHT, bool pluggedIn = false );
        QRectF plugBoundingRect() const;

        QPointer<Plug> m_plug;

        QPointF m_plugPosition;
        Orientation m_plugOrientation;

        bool m_movingPlug;
    };
}

#endif

