/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _KONDUIT_DATA_DUPLEXER_H_
#define _KONDUIT_DATA_DUPLEXER_H_

#include "switchelement.h"
#include "konduit_export.h"

#include <QtCore/QString>

namespace Konduit {
    
    class Socket;

    namespace Elements {

        /**
         * Duplicates a data stream on two sockets.
         */
        class KONDUIT_EXPORT DataDuplexer : public SwitchElement
        {
        public:
            DataDuplexer( QObject* parent, const QVariantList& args );
            virtual ~DataDuplexer();

            Socket* firstSocket() const;
            Socket* secondSocket() const;

            Soprano::StatementIterator data( const Socket* ) const;

            void paintInterface( QPainter* painter,
                                 const QStyleOptionGraphicsItem* option,
                                 const QRect& contentsRect );

        private:
            QSizeF contentsSizeHint( Qt::SizeHint which, const QSizeF& constraint ) const;

            class Private;
            Private* const d;
        };
    }
}

#endif
