/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "fileinputelement.h"

#include <kio/netaccess.h>

#include <Soprano/Parser>
#include <Soprano/Global>
#include <Soprano/RdfSerialization>
#include <Soprano/PluginManager>
#include <Soprano/StatementIterator>


class Konduit::Elements::FileInputElement::Private
{
public:
};


Konduit::Elements::FileInputElement::FileInputElement( QObject* parent, const QVariantList& args )
    : AbstractParseElement( parent, args ),
      d( 0 )
{
    ConfigurationOption o( "url", QVariant::Url );
    o.setLabel( i18n( "Url" ) );
    addConfigurationOption( o );
    setToolTip("File Input");
    resize( QSizeF(300, 200) );
    setMinimumSize( QSizeF(200, 100) );
}


Konduit::Elements::FileInputElement::~FileInputElement()
{
}


void Konduit::Elements::FileInputElement::setUrl( const KUrl& url )
{
    setOption( "url", ConfigurationSetting( url ) );
}


KUrl Konduit::Elements::FileInputElement::url() const
{
    return optionSetting( "url" ).value().toUrl();
}


Soprano::StatementIterator Konduit::Elements::FileInputElement::parseData( const Soprano::Parser* parser ) const
{
    QString temp;
    if ( KIO::NetAccess::download( url(), temp, 0 ) ) {
        Soprano::StatementIterator it = parser->parseFile( temp, baseUri(), serialization(), userSerialization() );
        KIO::NetAccess::removeTempFile( temp ); // FIXME: we cannot be sure that the parser does not need the file anymore!
        return it;
    }
    else {
        kDebug() << "failed to download" << url();
        return Soprano::StatementIterator();
    }
}

#include "fileinputelement.moc"
