/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _KONDUIT_FILE_INPUT_ELEMENT_H_
#define _KONDUIT_FILE_INPUT_ELEMENT_H_

#include "sinkelement.h"
#include "konduit_export.h"

#include <Soprano/RdfSerialization>

#include <kurl.h>

namespace Konduit {
    namespace Elements {
        /**
         * Writes data to a file.
         */
        class KONDUIT_EXPORT FileOutputElement : public SinkElement
        {
            Q_OBJECT

        public:
            FileOutputElement( QObject* parent, const QVariantList& args );
            virtual ~FileOutputElement();

            void setUrl( const KUrl& url );
            KUrl url() const;

            void setSerialization( Soprano::RdfSerialization s, const QString& userSerialization = QString() );
            Soprano::RdfSerialization serialization() const;
            QString userSerialization() const;

            /**
             * Save the data on plug
             */
            bool save( const Plug* plug );

        private:
            class Private;
            Private* const d;
        };
    }
}

#endif
