/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "inputelement.h"
#include "socket.h"
#include "plug.h"

#include <Soprano/Statement>
#include <Soprano/StatementIterator>

#include <QtCore/QQueue>
#include <QtCore/QDebug>
#include <QtGui/QPen>
#include <QtGui/QPainter>
#include <QGraphicsLinearLayout>

#include <plasma/widgets/lineedit.h>


class Konduit::Elements::InputElement::Private
{
public:
    Private( InputElement* parent )
        : q( parent ) {
    }

    Plasma::LineEdit *nameEdit;

private:
    InputElement* q;
};


Konduit::Elements::InputElement::InputElement( QObject* parent, const QVariantList& args )
    : SwitchElement( parent, args ),
      d( new Private( this ) )
{
    addPlug( "input" );
    addSocket( "output" );
    QGraphicsLinearLayout* lay = new QGraphicsLinearLayout( Qt::Vertical, this);

    ConfigurationOption o( "name", QVariant::String, "<name>" );
    o.setLabel( i18n( "name" ) );
    addConfigurationOption( o );

    d->nameEdit = new Plasma::LineEdit(this);
    lay->addItem(d->nameEdit);//, Plasma::NodeLayout::NodeCoordinate( 0, 0, 40, 10 ) );

    d->nameEdit->setText(getName());



    connect( this, SIGNAL( configurationValueChanged( const QString& ) ), this, SLOT( configChanged() ) );
    connect( d->nameEdit, SIGNAL( textEdited( const QString& ) ), this, SLOT( name_textChanged() ) );

    setToolTip("Input");;
    resize( QSizeF(130, 80) );
    setMinimumSize( QSizeF(90, 70) );
}



Konduit::Elements::InputElement::~InputElement()
{
    delete d;
}


Soprano::StatementIterator Konduit::Elements::InputElement::data( const Socket* s ) const
{
    Q_UNUSED( s )

    if ( !plug( "input" )->socket() ) {
        qDebug() << "InputElement::data: not connected to the input";
        return Soprano::StatementIterator();
    }

    return plug( "input" )->socket()->data();
}

/*
QSizeF Konduit::Elements::InputElement::sizeHint( Qt::SizeHint, const QSizeF& ) const
{
    // FIXME: this is just a size that is based on the socket and plug sizes
    // in an ideal world they would not be fixed
    return QSizeF( 130, 150 );
}
*/

/*
void Konduit::Elements::InputElement::paintInterface( QPainter* painter,
                                                    const QStyleOptionGraphicsItem* option,
                                                    const QRect& contentsRect )
{
    Element::paintInterface(painter, option, contentsRect);
    painter->fillRect(QRectF(0,0,130,50), QBrush(Qt::white));

    painter->setPen( QPen( Qt::black, 2 ) );
    painter->drawLine( 0, 10, 20, 10 );
    painter->drawLine( 0, 40, 20, 40 );
    painter->drawLine( 20, 10, 20, 40 );

    //draw an I to indicate an input element
    painter->drawLine( 30, 16, 30, 30 );

}
*/


void Konduit::Elements::InputElement::name_textChanged()
{
    blockSignals( true );
    setName( d->nameEdit->text() );
    blockSignals( false );
}


void Konduit::Elements::InputElement::configChanged()
{
    d->nameEdit->setText(getName());
}


QString Konduit::Elements::InputElement::getName()
{
    return optionSetting( "name" ).value().toString();
}


void Konduit::Elements::InputElement::setName(const QString& _name)
{
    setOption( "name", ConfigurationSetting( _name ) );
}


#include "inputelement.moc"
