/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _KONDUIT_STORAGE_MODEL_H_
#define _KONDUIT_STORAGE_MODEL_H_

#include <Soprano/FilterModel>

#include "konduit_export.h"

namespace Soprano {
    class Model;
}

namespace Konduit {

    class Element;

    /**
     * Soprano FilterModel to save and load konduits.
     */
    class KONDUIT_EXPORT StorageModel : public Soprano::FilterModel
    {
    public:
        StorageModel();
        ~StorageModel();

        QUrl findKonduit( const QString& name ) const;
        QUrl findKonduitBlackBox( const QString& name ) const;
        QUrl findKonduitMain( const QString& name, bool isBlackBox ) const;

        QUrl saveKonduit( Element* someElement, const QString& name );
        QUrl saveKonduitBlackBox( Element* someElement, const QString& name, const QString& comment, const QString& category );
        QUrl saveKonduitMain( Element* someElement, const QString& name, bool isBlackBox );

        /**
         * Loads a Konduit.
         *
         * \return An arbitrary Element from the Konduit or
         * 0 if the konduit could not be loaded.
         */
        Element* loadKonduit( const QString& name );
        Element* loadKonduitBlackBox( const QString& name );
        Element* loadKonduitMain( const QString& name, bool isBlackBox );

        /**
         * Checks names and connectedness of inputs/outputs
         */
        bool validateBlackBox( Element* someElement );
        
        /**
         * Removes a black box from the repository and the filesystem
         */
        bool deleteBlackBox( const QString& name );
         
        /**
         * Removes a konduit from the repository
         */
        bool removeKonduit( const QString& name, bool isBlackBox );
        
        /**
         * Exports the given black box to the given file (using an internal file type)
         */
        bool exportBlackBox( const QString& name, const QString& fileName );
        
        /**
         * Imports the given black box from the given file
         */
        bool importBlackBox( const QString& fileName) ;

        /**
         * \return A list of the names of all saved konduits.
         */
        QStringList allSavedKonduits();
        QStringList allSavedKonduitBlackBoxes();
        QStringList allSavedKonduitsMain(bool isBlackBox);
        
        /**
         * Constructs a path to a plugin in the local services directory
         */
        QString pluginPath( const QString& pluginName);
        
        /**
         * Checks if the blackbox described in a file exists in the repository, and returns its name, if it does.
         */
        QString findBlackboxFromFile( const QString& filePath );

    private:
        bool saveDesktopFile( const QString& filePath, const QString& name, const QString& comment, const QString& category );
    
        Soprano::Model* m_sopranoModel;
        
    };
}

#endif
