/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "lineedit.h"

#include <QCompleter>
#include <QKeyEvent>
#include <QAbstractItemView>
#include <QtDebug>
#include <QApplication>
#include <QModelIndex>
#include <QAbstractItemModel>
#include <QStandardItemModel>
#include <QStringListModel>
#include <QStandardItem>
#include <QScrollBar>
#include <QModelIndex>

#include "combobox.h"

#include <KDebug>

LineEdit::LineEdit(ComboBox* _cb, QWidget *parent)
: QLineEdit(parent), c(0), cb(_cb)
{
}

LineEdit::~LineEdit()
{
}

void LineEdit::setCompleter(QCompleter *completer)
{
    if (c && c!=completer) {
        QObject::disconnect(c, 0, this, 0);
        c->popup()->hide();
    }

    c = completer;

    if (!c)
        return;

    c->setWidget(this);
    c->setCompletionMode(QCompleter::UnfilteredPopupCompletion);
    c->setCaseSensitivity(Qt::CaseInsensitive);
    c->popup()->setAlternatingRowColors( true );
    QObject::connect(completer, SIGNAL(activated(const QString&)),
                  this, SLOT(setText(const QString&)));
}

QCompleter *LineEdit::completer() const
{
    return c;
}

void LineEdit::setExtraCompletion( QStringList compList )
{
    extraCompletionList << compList;
    //kDebug() << "Setting extraCompletionList to " << compList;
}


void LineEdit::keyPressEvent(QKeyEvent *e)
{
   if( e->key() == Qt::Key_Backspace ) {
       backspace();
       e->accept();
   }
   else if (e->key() == Qt::Key_Delete) {
       del();
       e->accept();
   }
   else if ( e->key() == Qt::Key_Down ) {
       QLineEdit::keyPressEvent( e );
       e->accept();
   }
   else if( e->key() & 0x01000000 ) { //all other non-text keys
       QLineEdit::keyPressEvent( e );
       return; // let the completer do default behavior
   }
   else {
       //setText( text() + e->text() );
       QLineEdit::keyPressEvent( e );
       e->accept();
   }

    //build model from predicates
    QStringListModel* qsim = new QStringListModel(c);
    QStringList qsl;
    
    //construct list of all the items for the autocompletion
    QStringList allItems = cb->predicates();
    allItems << extraCompletionList;
    
    foreach( QString p, allItems.toSet().toList() ) {
        if( p.startsWith( text() ) || p.contains( ":" + text() ) ) {
            qsl << p;
        }
    }
    qsim->setStringList( qsl );
    
    //create completer with empty prefix, and complete
    LineEdit::setCompleter( new QCompleter( qsim, this ) );
    c->setCompletionPrefix( "" );
    c->popup()->setCurrentIndex(c->completionModel()->index(0, 0));
    c->complete();
}

#include "lineedit.moc"
