/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _KONDUIT_LITERAL_ELEMENT_H_
#define _KONDUIT_LITERAL_ELEMENT_H_

#include "sourceelement.h"
#include "konduit_export.h"

#include <kurl.h>

namespace Soprano {
    class LiteralValue;
}

namespace Konduit {
    namespace Elements {
        /**
         * A literal element provides a single literal value as data.
         *
         * Subclasses need to implement literalData() instead of data().
         */
        class KONDUIT_EXPORT LiteralElement : public SourceElement
        {
            Q_OBJECT

        public:
            LiteralElement( QObject* parent, const QVariantList& args );
            virtual ~LiteralElement();

            Soprano::StatementIterator data( const Socket* socket ) const;

        protected:
            virtual Soprano::LiteralValue literalData() const = 0;

        private:
            class Private;
            Private* const d;
        };
    }
}

#endif
