/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "manager.h"
#include "element.h"

Konduit::Manager::Manager()
    : QObject( 0 )
{
}


Konduit::Manager::~Manager()
{
}


int Konduit::Manager::generateGroupId()
{
    static int s_gidc = 0;
    return ++s_gidc;
}


QList<int> Konduit::Manager::allGroups() const
{
    return m_groups.keys();
}


Konduit::Element* Konduit::Manager::groupItem( int groupId ) const
{
    if ( m_groups.contains( groupId ) ) {
        return m_groups[groupId];
    }
    else {
        return 0;
    }
}


bool Konduit::Manager::isFullyPlugged( int groupId )
{
    Element* e = groupItem( groupId );
    if ( e ) {
        return e->isFullyPlugged();
    }
    else {
        return false;
    }
}


Q_GLOBAL_STATIC( Konduit::Manager, manager )

Konduit::Manager* Konduit::Manager::instance()
{
    return manager();
}


void Konduit::Manager::updateGrouping( Element* element )
{
    // remove the Element's old group entry
    QList<int> gl = m_groups.keys( element );
    Q_FOREACH( int group, gl ) {
        m_groups.remove( group );
    }

    // add the new group thing
    m_groups[element->groupId()] = element;

    emit groupingChanged();
}

#include "manager.moc"
