/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _KONDUIT_MANAGER_H_
#define _KONDUIT_MANAGER_H_

#include <QtCore/QObject>
#include <QtCore/QHash>

#include "element.h"

namespace Konduit {
    class Manager : public QObject
    {
    Q_OBJECT

    public:
    Manager();
    ~Manager();

    int generateGroupId();

    QList<int> allGroups() const;

    /**
     * Retrieve one item representing a group. Other
     * items of the group can be retrieved by traversing
     * the connections.
     */
    Element* groupItem( int groupId ) const;

    bool isFullyPlugged( int groupId );

    void updateGrouping( Element* );

    static Manager* instance();

    Q_SIGNALS:
    void groupingChanged();

    private:
    // contains an arbitrary Element for each group
    QHash<int,Element*> m_groups;
    };
}

#endif
