/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "memorymodelelement.h"
#include "plug.h"
#include "socket.h"

#include <Soprano/StorageModel>
#include <Soprano/StatementIterator>
#include <Soprano/Backend>
#include <Soprano/Global>
#include <Soprano/PluginManager>

#include <KDebug>


class Konduit::Elements::MemoryModelElement::Private
{
public:
    Private()
        : backend( 0 ) {
    }
    const Soprano::Backend* backend;
};

Konduit::Elements::MemoryModelElement::MemoryModelElement( QObject* parent, const QVariantList& args )
    : SinkElement( parent, args ),
      d( new Private() )
{
}


Konduit::Elements::MemoryModelElement::~MemoryModelElement()
{
    delete d;
}


Soprano::Model* Konduit::Elements::MemoryModelElement::model() const
{
    if ( !d->backend ) {
        d->backend = Soprano::PluginManager::instance()->discoverBackendByFeatures( Soprano::BackendFeatureStorageMemory );
    }

    if ( d->backend ) {
        Soprano::BackendSettings settings;
        settings.append( Soprano::BackendSetting( Soprano::BackendOptionStorageMemory ) );
        Soprano::Model* model = d->backend->createModel( settings );
        if ( model ) {
            QList<Plug*> plugs = dynamic_cast<const Sink*>( this )->allPlugs();
            Q_FOREACH( Plug* plug, plugs ) {
                if ( plug->socket() ) {
                    Soprano::StatementIterator it = plug->socket()->data();
                    while ( it.next() ) {
                        model->addStatement( *it );
                    }
                }
            }
        }
        return model;
    }

    return 0;
}


Soprano::Model* Konduit::Elements::MemoryModelElement::model( Plug* plug ) const
{
    if ( plug ) {
        if ( !d->backend ) {
            d->backend = Soprano::PluginManager::instance()->discoverBackendByFeatures( Soprano::BackendFeatureStorageMemory );
        }

        if ( d->backend ) {
            Soprano::BackendSettings settings;
            settings.append( Soprano::BackendSetting( Soprano::BackendOptionStorageMemory ) );
            Soprano::Model* model = d->backend->createModel( settings );
            if ( model ) {
                if ( plug->socket() ) {
                    Soprano::StatementIterator it = plug->socket()->data();
                    while ( it.next() ) {
                        model->addStatement( *it );
                    }
                }
                return model;
            }
        }
    }
    else {
        return model();
    }

    return 0;
}

#include "memorymodelelement.moc"
