/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _KONDUIT_MEMORY_MODEL_ELEMENT_H_
#define _KONDUIT_MEMORY_MODEL_ELEMENT_H_

#include "sinkelement.h"
#include "konduit_export.h"

namespace Konduit {
    namespace Elements {

        /**
         * Combines all its inputs into one memory model.
         *
         * See also \ref konduit_memory_model_element
         */
        class KONDUIT_EXPORT MemoryModelElement : public SinkElement
        {
            Q_OBJECT

        public:
            virtual ~MemoryModelElement();

        protected:
            MemoryModelElement( QObject* parent, const QVariantList& args );

            /**
             * \return A Model containing the data from all input Plugs
             * or 0 on error.
             */
            Soprano::Model* model() const;

            /**
             * \return A Model containing the data from Plug \p plug
             * or 0 on error.
             */
            Soprano::Model* model( Plug* plug ) const;

        private:
            class Private;
            Private* const d;
        };
    }
}

#endif
