/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _KONDUIT_PLUG_H_
#define _KONDUIT_PLUG_H_

#include <QtCore/QObject>
#include <QtCore/QString>
#include <QtGui/QGraphicsItem>

#include "konduit_export.h"

namespace Soprano {
    class StatementIterator;
}


namespace Konduit {

    class Sink;
    class Socket;

    /**
     * A Sink can have multiple plugs. Each Plug
     * can be connected to a Socket.
     */
    class KONDUIT_EXPORT Plug : public QObject, public QGraphicsItem
    {
        Q_OBJECT

    public:
        ~Plug();

        /**
         * The name of the plug.
         */
        QString name() const;

        /**
         * The Sink this plug belongs to.
         */
        Sink* sink() const;

        /**
         * Connect socket to this plug.
         * The parent Sink will read data from socket.
         */
        bool plugIn( Socket* socket );

        void unplug();

        Socket* socket() const;

        QRectF boundingRect() const;
        void paint( QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget = 0 );

        void updatePosition();

        /**
         * Retrive the stream of data from the socket we are plugged into.
         * If we are not plugged in, an invalid itererator will be returned.
         * 
         * \sa Socket::data
         */
        Soprano::StatementIterator data() const;

    Q_SIGNALS:
        void plugStateChanged( Konduit::Socket* oldSocket, Konduit::Socket* newSocket );

    private:
        Plug( Sink* sink, const QString& name = QString() );

        QVariant itemChange( GraphicsItemChange change, const QVariant& value );

        class Private;
        Private* const d;

        // Plug instances are owned by their parent Sink
        friend class Sink;
    };
}

#endif
