/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "pluginicon.h"
#include "plug.h"
#include "socket.h"
#include "sink.h"
#include "cord.h"
#include "element.h"

#include <QtGui/QPainter>
#include <QtGui/QGraphicsScene>
#include <QtCore/QPointer>

#include <Soprano/StatementIterator>

#include <KDebug>
#include <KIcon>


class Konduit::PluginIcon::Private
{
public:
    Private( Element* _parent )
        : parent( _parent ),
          boundingRect( 0, 0, 20, 20 )
    {
    }

    Element* parent;
    QRectF boundingRect;
};


Konduit::PluginIcon::PluginIcon( Element* parent )
    : QObject( 0 ),
      QGraphicsItem( dynamic_cast<QGraphicsItem*>( parent )->parentItem() ),
      d( new Private( parent ) )
{
    setZValue( dynamic_cast<QGraphicsItem*>( parent )->zValue()+1 );
    setToolTip( parent->name() );
}


Konduit::PluginIcon::~PluginIcon()
{
    delete d;
}


QRectF Konduit::PluginIcon::boundingRect() const
{
    return d->boundingRect;
}


void Konduit::PluginIcon::paint( QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget )
{
    Q_UNUSED( option );
    Q_UNUSED( widget );
    
    painter->fillRect( 0, 0, 20, 20, Qt::white );
    painter->drawRect( 0, 0, 20, 20 );
    KIcon( d->parent->icon() ).paint( painter, 0, 0, 20, 20 );
}


QVariant Konduit::PluginIcon::itemChange( GraphicsItemChange change, const QVariant& value )
{
    return QGraphicsItem::itemChange( change, value );
}


#include "pluginicon.moc"
