/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "datadumpelement.h"
#include "plug.h"
#include "source.h"
#include "socket.h"

#include <Soprano/StatementIterator>

#include <QtCore/QTextStream>
#include <QtGui/QTextBrowser>
#include <QGraphicsLinearLayout>

#include <plasma/widgets/label.h>
#include <plasma/widgets/pushbutton.h>


class Konduit::Elements::DataDumpElement::Private
{
public:
    Plasma::Label* view;
    QTextBrowser* browser;

    void init( DataDumpElement* e ) {
        e->addPlug( "input" );

        browser = new QTextBrowser( 0 );

        QGraphicsLinearLayout* mainLay = new QGraphicsLinearLayout( Qt::Vertical, e);

        Plasma::PushButton* startButton = new Plasma::PushButton( e );
        startButton->setText( "Start" );

        mainLay->addItem( startButton );
        connect( startButton, SIGNAL( clicked() ), e, SLOT( start() ) );
    }
};


Konduit::Elements::DataDumpElement::DataDumpElement( QGraphicsItem* parent, const QString& serviceId, uint appletId )
    : SinkElement( parent, serviceId, appletId ),
      d( new Private() )
{
    d->init( this );
}


Konduit::Elements::DataDumpElement::DataDumpElement( QObject* parent, const QVariantList& args )
    : SinkElement( parent, args ),
      d( new Private() )
{
    d->init( this );
    setToolTip("Data Dump");
    resize( QSizeF(130, 80) );
    setMinimumSize( QSizeF(90, 60) );
}


Konduit::Elements::DataDumpElement::~DataDumpElement()
{
    delete d;
}


bool Konduit::Elements::DataDumpElement::start()
{
    kDebug();
    // get the data from the source and dump it
    Plug* p = plug( "input" );
    if ( p->socket() ) {
        QTextStream s( stdout );
        QString buf;
        QTextStream bufS( &buf );
        Soprano::StatementIterator it = p->socket()->data();
        while ( it.next() ) {
//            s << *it << endl;
            bufS << *it << endl;
        }
//        d->view->setText( buf );
        d->browser->setPlainText( buf );
        d->browser->show();
        return true;
    }
    else {
        return false;
    }
}

#include "datadumpelement.moc"
