/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "datainserterelement.h"
#include "plug.h"
#include "source.h"
#include "socket.h"

#include <Soprano/StatementIterator>
#include <Soprano/Statement>
#include <Nepomuk/ResourceManager>
#include <Soprano/Model>

#include <QtCore/QTextStream>
#include <QGraphicsLinearLayout>

#include <plasma/widgets/label.h>
#include <plasma/widgets/pushbutton.h>

#include <KMessageBox>

class Konduit::Elements::DataInserterElement::Private
{
public:
    Plasma::Label* view;

    void init( DataInserterElement* e ) {
        e->addPlug( "input" );

        QGraphicsLinearLayout* mainLay = new QGraphicsLinearLayout( Qt::Vertical, e);

        Plasma::PushButton* startButton = new Plasma::PushButton( e );
        startButton->setText( "Insert" );

        mainLay->addItem( startButton );
        connect( startButton, SIGNAL( clicked() ), e, SLOT( start() ) );
    }
};


Konduit::Elements::DataInserterElement::DataInserterElement( QGraphicsItem* parent, const QString& serviceId, uint appletId )
    : SinkElement( parent, serviceId, appletId ),
      d( new Private() )
{
    d->init( this );
}


Konduit::Elements::DataInserterElement::DataInserterElement( QObject* parent, const QVariantList& args )
    : SinkElement( parent, args ),
      d( new Private() )
{
    d->init( this );
    setToolTip("Nepomuk Data Inserter");
    resize( QSizeF(130, 80) );
    setMinimumSize( QSizeF(90, 60) );
}


Konduit::Elements::DataInserterElement::~DataInserterElement()
{
    delete d;
}


bool Konduit::Elements::DataInserterElement::start()
{
    Plug* p = plug( "input" );
    if ( p->socket() ) {
        Soprano::StatementIterator it = p->socket()->data();
        Soprano::Model* main = nepomukMainModel();
        QList<Soprano::Statement> allStatements = it.allStatements();
        QTime t;
        t.start();
        int i = 0;
        
        foreach( Soprano::Statement s, allStatements ) {
            if (!main->containsAnyStatement( s.subject(), s.predicate(), s.object() )) {
                main->addStatement( s );
                kDebug() << "Added statement to Nepomuk: " << s;
                i++;
            }
        }
        
        KMessageBox::information( 0, i18n( "Added %1 statements in %2 milliseconds.",  i, t.elapsed() ) );
        
        return true;
    }
    else {
        return false;
    }
}

#include "datainserterelement.moc"
