/*
 * This file is part of Konduit Project.
 *
* Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _KONDUIT_DATATABLE_ELEMENT_H_
#define _KONDUIT_DATATABLE_ELEMENT_H_

#include "sinkelement.h"

#include <Soprano/StatementIterator>

#include "konduit_export.h"

namespace Konduit {
    namespace Elements {
    /**
     * A simple sink element that prints the data in tables to QTextBrowser windows
     */
    class KONDUIT_EXPORT DataTableElement : public SinkElement
    {
        Q_OBJECT

    public:
        DataTableElement( QGraphicsItem* parent, const QString& serviceId, uint appletId );
        DataTableElement( QObject* parent, const QVariantList& args );
        virtual ~DataTableElement();

    public Q_SLOTS:
        bool start();

    private:
        class Private;
        Private* const d;
        //checks whether s is on column c of t /\ returns the row
        int onColumn(QString s, int c, QList<QList <QString> > t);
        void displaySingleText(QList<Soprano::Statement> allStatements);
        void displaySingleTable(QList<Soprano::Statement> allStatements);
        void displaySeparateTables(QList<Soprano::Statement> allStatements);
        QString getRDFLabel(QString resource);

    };
    }
}

K_EXPORT_PLASMA_APPLET(konduit_test, Konduit::Elements::DataTableElement)

#endif
