/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "duplicateremover.h"
#include "socket.h"
#include "plug.h"

//#include <soprano.h>
#include <soprano/soprano.h>
#include <Soprano/Backend>
#include <Soprano/Model>

#include <Soprano/Statement>
#include <Soprano/StatementIterator>

#include <QtCore/QQueue>
#include <QtCore/QDebug>

#include <QList>
#include <QSet>
#include <QHash>


class Konduit::Elements::DuplicateRemover::Private
{
public:
    Private( DuplicateRemover* parent )
        : q( parent ) {
    }


private:
    DuplicateRemover* q;
};


Konduit::Elements::DuplicateRemover::DuplicateRemover( QObject* parent, const QVariantList& args )
    : SwitchElement( parent, args ),
      d( new Private( this ) )
{
    addPlug( "input" );
    addSocket( "output" );

    setToolTip("Duplicate Remover");
    resize( QSizeF(70, 100) );
    setMinimumSize( QSizeF(50, 75) );
}



Konduit::Elements::DuplicateRemover::~DuplicateRemover()
{
    delete d;
}


Soprano::StatementIterator Konduit::Elements::DuplicateRemover::data( const Socket* s ) const
{
    Q_UNUSED( s )

    if ( !plug( "input" )->socket() ) {
        qDebug() << "DuplicateRemover::data: not connected to the input";
        return Soprano::StatementIterator();
    }

    return dataClean( ); //plug( "input" )->socket()->data();
}


Soprano::StatementIterator Konduit::Elements::DuplicateRemover::dataClean( ) const
{
    Plug* p = plug( "input" );
    Soprano::StatementIterator it = p->socket()->data();

    QList<Soprano::Statement> allStatements = it.allElements();
    //QSet<Soprano::Statement> allStatementsClean = QSet<Soprano::Statement>::fromList( allStatements );//.toSet();
    QList<Soprano::Statement> cleanStatements;

    foreach(Soprano::Statement st, allStatements) {
        if( !cleanStatements.contains( st ) ) {
             cleanStatements.append( st );
        }
    }

    Soprano::Model* model = Soprano::createModel();
    model->addStatements( cleanStatements );
 
    return model->listStatements(); 
}

/*
QSizeF Konduit::Elements::DuplicateRemover::sizeHint( Qt::SizeHint, const QSizeF& ) const
{
    // FIXME: this is just a size that is based on the socket and plug sizes
    // in an ideal world they would not be fixed
    return QSizeF( 130, 150 );
}
*/

/*
void Konduit::Elements::DuplicateRemover::paintInterface( QPainter* painter,
                                                    const QStyleOptionGraphicsItem* option,
                                                    const QRect& contentsRect )
{
    Element::paintInterface(painter, option, contentsRect);
    painter->fillRect(QRectF(0,0,130,50), QBrush(Qt::white));

    painter->setPen( QPen( Qt::black, 2 ) );
    painter->drawLine( 0, 10, 20, 10 );
    painter->drawLine( 0, 40, 20, 40 );
    painter->drawLine( 20, 10, 20, 40 );

    //draw an I to indicate an input element
    painter->drawLine( 30, 16, 30, 30 );

}
*/

#include "duplicateremover.moc"
