/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _KONDUIT_DUPLICATE_REMOVER_H_
#define _KONDUIT_DUPLICATE_REMOVER_H_

#include "switchelement.h"
#include "konduit_export.h"

#include <QtCore/QString>

namespace Konduit {
    
    class Socket;

    namespace Elements {

        /**
         * Removes duplicates from the input RDF graph
         *
         * Defines the plug "input", and output "data".
         */
        class KONDUIT_EXPORT DuplicateRemover : public SwitchElement //inherits Element, Sink and Source
        {
        Q_OBJECT

        public:
            DuplicateRemover( QObject* parent, const QVariantList& args );
            virtual ~DuplicateRemover();

            Soprano::StatementIterator data( const Socket* ) const;

            /*void paintInterface( QPainter* painter,
                                 const QStyleOptionGraphicsItem* option,
                                 const QRect& contentsRect );
            */


        private:
            //QSizeF sizeHint( Qt::SizeHint which, const QSizeF& constraint ) const;

            class Private;
            Private* const d;

            /**
              * Removes duplicates
              */  
            Soprano::StatementIterator dataClean( ) const;

        };
    }
}

K_EXPORT_PLASMA_APPLET(konduit_duplicateremover, Konduit::Elements::DuplicateRemover)

#endif
