/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2008 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "filelistinputelement.h"
#include "configurationoption.h"

#include <KLocale>
#include <KUrl>

#include <Soprano/Statement>
#include <Soprano/Util/SimpleStatementIterator>
#include <Soprano/Vocabulary/RDF>
#include <Soprano/Vocabulary/Xesam>


Konduit::Elements::FileListInputElement::FileListInputElement( QObject* parent, const QVariantList& args )
    : SourceElement( parent, args )
{
    addSocket( "files" );

    ConfigurationOption option( "files", QVariant::Url );
    option.setIsListOption( true );
    option.setLabel( i18n( "Files" ) );
    addConfigurationOption( option );

    resize( QSizeF(200, 150) );
    setMinimumSize( QSizeF(150, 105) );

    createStandardGui();

    setToolTip("File List Input");
}


void Konduit::Elements::FileListInputElement::slotFindFiles()
{
}


QList<KUrl> Konduit::Elements::FileListInputElement::fileUrls() const
{
    QList<KUrl> urls;
    ConfigurationSetting setting = optionSetting( "files" );
    foreach( QVariant v, setting.values() ) {
        urls << v.toUrl();
    }
    return urls;
}


Soprano::StatementIterator Konduit::Elements::FileListInputElement::data( const Socket* ) const
{
    QList<Soprano::Statement> sl;
    foreach( KUrl url, fileUrls() ) {
        sl << Soprano::Statement( url, Soprano::Vocabulary::RDF::type(), Soprano::Vocabulary::Xesam::File() )
           << Soprano::Statement( url, Soprano::Vocabulary::Xesam::url(), Soprano::LiteralValue( url.url() ) );
    }
    return Soprano::Util::SimpleStatementIterator( sl );
}

#include "filelistinputelement.moc"
