/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "fileliteralelement.h"

#include <plasma/widgets/pushbutton.h>
#include <plasma/widgets/label.h>
#include <plasma/widgets/lineedit.h>

#include <kfiledialog.h>
#include <kio/netaccess.h>

#include <KLocale>

#include <Soprano/LiteralValue>

#include <QGraphicsProxyWidget>
#include <QGraphicsLinearLayout>
#include <KPushButton>
#include <QPalette>
#include <QLabel>


Konduit::Elements::FileLiteralElement::FileLiteralElement( QObject* parent, const QVariantList& args )
    : LiteralElement( parent, args )
{
    addConfigurationOption( ConfigurationOption( "url", QVariant::String ) );

    Plasma::Label* label = new Plasma::Label( this );
    label->setText( i18n( "Read literal data from a file:" ) );
    label->nativeWidget()->setStyleSheet(" QLabel { background-color:#ffffff; color:#000000; } " );
    m_urlLabel = new Plasma::Label( this );
    m_urlLabel->nativeWidget()->setStyleSheet(" QLabel { background-color:#ffffff; color:#000000; } " );

    Plasma::PushButton* button = new Plasma::PushButton( this );
    button->nativeWidget()->setText( "Open" ); //FIXME: add Open icon, like setIcon( "document-open" )

    QGraphicsLinearLayout* mainLayout = new QGraphicsLinearLayout( Qt::Vertical, this);
    QGraphicsLinearLayout* urlLayout = new QGraphicsLinearLayout( );

    mainLayout->addItem( label );
    urlLayout->addItem( m_urlLabel );
    urlLayout->addItem( button );
    mainLayout->addItem( urlLayout );

    connect( button, SIGNAL( clicked() ), this, SLOT( slotFindFile() ) );
    connect( this, SIGNAL( configurationValueChanged( const QString& ) ),
             this, SLOT( slotConfigChanged() ) );

    setToolTip("File Literal");
    resize( QSizeF(200, 100) );
    setMinimumSize( QSizeF(120, 90) );
}


Konduit::Elements::FileLiteralElement::~FileLiteralElement()
{
}


void Konduit::Elements::FileLiteralElement::setUrl( const KUrl& url )
{
    setOption( "url", ConfigurationSetting( url.url() ) );
}


KUrl Konduit::Elements::FileLiteralElement::url() const
{
    return optionSetting( "url" ).value().toString();
}


void Konduit::Elements::FileLiteralElement::slotFindFile()
{
    KUrl url = KFileDialog::getOpenUrl();
    if ( !url.isEmpty() ) {
        setUrl( url );
    }
}


void Konduit::Elements::FileLiteralElement::slotConfigChanged()
{
    m_urlLabel->setText( url().url() );
    updateGeometry();
}


Soprano::LiteralValue Konduit::Elements::FileLiteralElement::literalData() const
{
    QString temp;
    QString literal;
    if ( KIO::NetAccess::download( url(), temp, 0 ) ) {
        QFile f( temp );
        if ( f.open( QIODevice::ReadOnly ) ) {
            literal = QTextStream( &f ).readAll();
        }
        KIO::NetAccess::removeTempFile( temp ); // FIXME: we cannot be sure that the parser does not need the file anymore!
    }
    return literal;
}

#include "fileliteralelement.moc"
