/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2008 Sebastian Trueg <trueg@kde.org>
 * Copyright (C) 2008 Laura Dragan <aprilush@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "fileopenelement.h"
#include "plug.h"
#include "source.h"
#include "socket.h"
#include <KFileItem>
#include <Soprano/QueryResultIterator>
#include <Soprano/Model>
#include <Soprano/Node>
#include <Soprano/Vocabulary/RDF>
#include <Soprano/Vocabulary/Xesam>
#include <Soprano/StatementIterator>
#include <QGraphicsLinearLayout>

class Konduit::Elements::FileOpenElement::Private
{
    public:
        void init(FileOpenElement* e) {
            e->addPlug("files");
            QGraphicsLinearLayout* mainLay = new QGraphicsLinearLayout(Qt::Vertical, e);
            Plasma::PushButton* startButton = new Plasma::PushButton(e);
            startButton->setText("Open");
            mainLay->addItem(startButton);
            connect(startButton, SIGNAL(clicked()), e, SLOT(start()));
        }
};

Konduit::Elements::FileOpenElement::FileOpenElement(QObject* parent, const QVariantList& args) :
    MemoryModelElement(parent, args) {
    setHasConfigurationInterface(false);

    addPlug("files");

    QGraphicsLinearLayout* mainLay = new QGraphicsLinearLayout(Qt::Vertical, this);
    m_openButton = new Plasma::PushButton(this);
    m_openButton->setText("Open");
    mainLay->addItem(m_openButton);

    connect(m_openButton, SIGNAL(clicked()), this, SLOT(openFiles()));

    setToolTip("Open files");
    resize(QSizeF(130, 80));
    setMinimumSize(QSizeF(90, 60));
}

Konduit::Elements::FileOpenElement::~FileOpenElement() {
    delete m_openButton;
}

bool Konduit::Elements::FileOpenElement::openFiles() {
    kDebug();
    Plug* p = plug("files");
    if (p->socket()) {
        QList<KUrl> urls;
        //        Soprano::StatementIterator it = plug("files")->data();
        //        while (it.next()) {
        //            Soprano::Statement s = *it;
        //            if (s.predicate() == Soprano::Vocabulary::Xesam::url()) {
        //                urls << s.object().toString();
        //            }
        //        }
        if (Soprano::Model * m = model(plug("files"))) {
            Soprano::QueryResultIterator it = m->executeQuery(QString("prefix xesam: <%1> "
                "select distinct ?url where { ?r a xesam:File . "
                "?r xesam:url ?url . }") .arg(
                            Soprano::Vocabulary::Xesam::xesamNamespace().toString()),
                            Soprano::Query::QueryLanguageSparql);
            while (it.next()) {
                urls << it.binding("url").toString();
            }
            delete m;
        } else {
            Soprano::StatementIterator it = plug("files")->data();
            while (it.next()) {
                Soprano::Statement s = *it;
                if (s.predicate() == Soprano::Vocabulary::Xesam::url()) {
                    urls << s.object().toString();
                }
            }
        }
        foreach (KUrl url , urls) {
            KFileItem item(KFileItem::Unknown, KFileItem::Unknown, url);
            item.run();
        }
        return true;
    } else {
        return false;
    }
}

#include "fileopenelement.moc"
