/*
 * This file is part of Soprano Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _SOPRANO_FOAF_H_
#define _SOPRANO_FOAF_H_

#include <QtCore/QUrl>

#define SOPRANO_EXPORT

namespace Soprano {
    namespace Vocabulary {
        namespace FOAF {
            /**
             * http://purl.org/dc/elements/1.1/
             */
            SOPRANO_EXPORT QUrl foafNamespace();

            /**
             * http://purl.org/dc/elements/1.1/date 
             */
            SOPRANO_EXPORT QUrl date();

            /**
             * http://purl.org/dc/elements/1.1/description 
             */
            SOPRANO_EXPORT QUrl description();

            /**
             * http://purl.org/dc/elements/1.1/title 
             */
            SOPRANO_EXPORT QUrl title();

            /**
             * http://www.w3.org/2000/01/rdf-schema#Class 
             */
            SOPRANO_EXPORT QUrl Class();

            /**
             * http://www.w3.org/2003/01/geo/wgs84_pos#SpatialThing 
             */
            SOPRANO_EXPORT QUrl SpatialThing();

            /**
             * http://www.w3.org/2003/06/sw-vocab-status/ns#term_status 
             */
            SOPRANO_EXPORT QUrl term_status();

            /**
             * http://xmlns.com/foaf/0.1/Agent 
             * 
             * An agent (eg. person, group, software or physical artifact). 
             */
            SOPRANO_EXPORT QUrl Agent();

            /**
             * http://xmlns.com/foaf/0.1/Document 
             * 
             * A document. 
             */
            SOPRANO_EXPORT QUrl Document();

            /**
             * http://xmlns.com/foaf/0.1/Group 
             * 
             * A class of Agents. 
             */
            SOPRANO_EXPORT QUrl Group();

            /**
             * http://xmlns.com/foaf/0.1/Image 
             * 
             * An image. 
             */
            SOPRANO_EXPORT QUrl Image();

            /**
             * http://xmlns.com/foaf/0.1/OnlineAccount 
             * 
             * An online account. 
             */
            SOPRANO_EXPORT QUrl OnlineAccount();

            /**
             * http://xmlns.com/foaf/0.1/OnlineChatAccount 
             * 
             * An online chat account. 
             */
            SOPRANO_EXPORT QUrl OnlineChatAccount();

            /**
             * http://xmlns.com/foaf/0.1/OnlineEcommerceAccount 
             * 
             * An online e-commerce account. 
             */
            SOPRANO_EXPORT QUrl OnlineEcommerceAccount();

            /**
             * http://xmlns.com/foaf/0.1/OnlineGamingAccount 
             * 
             * An online gaming account. 
             */
            SOPRANO_EXPORT QUrl OnlineGamingAccount();

            /**
             * http://xmlns.com/foaf/0.1/Organization 
             * 
             * An organization. 
             */
            SOPRANO_EXPORT QUrl Organization();

            /**
             * http://xmlns.com/foaf/0.1/Person 
             * 
             * A person. 
             */
            SOPRANO_EXPORT QUrl Person();

            /**
             * http://xmlns.com/foaf/0.1/PersonalProfileDocument 
             * 
             * A personal profile RDF document. 
             */
            SOPRANO_EXPORT QUrl PersonalProfileDocument();

            /**
             * http://xmlns.com/foaf/0.1/Project 
             * 
             * A project (a collective endeavour of some kind). 
             */
            SOPRANO_EXPORT QUrl Project();

            /**
             * http://xmlns.com/foaf/0.1/accountName 
             * 
             * Indicates the name (identifier) associated with this online 
             * account. 
             */
            SOPRANO_EXPORT QUrl accountName();

            /**
             * http://xmlns.com/foaf/0.1/accountServiceHomepage 
             * 
             * Indicates a homepage of the service provide for this online 
             * account. 
             */
            SOPRANO_EXPORT QUrl accountServiceHomepage();

            /**
             * http://xmlns.com/foaf/0.1/aimChatID 
             * 
             * An AIM chat ID 
             */
            SOPRANO_EXPORT QUrl aimChatID();

            /**
             * http://xmlns.com/foaf/0.1/based_near 
             * 
             * A location that something is based near, for some broadly human 
             * notion of near. 
             */
            SOPRANO_EXPORT QUrl based_near();

            /**
             * http://xmlns.com/foaf/0.1/birthday 
             * 
             * The birthday of this Agent, represented in mm-dd string form, 
             * eg. '12-31'. 
             */
            SOPRANO_EXPORT QUrl birthday();

            /**
             * http://xmlns.com/foaf/0.1/currentProject 
             * 
             * A current project this person works on. 
             */
            SOPRANO_EXPORT QUrl currentProject();

            /**
             * http://xmlns.com/foaf/0.1/depiction 
             * 
             * A depiction of some thing. 
             */
            SOPRANO_EXPORT QUrl depiction();

            /**
             * http://xmlns.com/foaf/0.1/depicts 
             * 
             * A thing depicted in this representation. 
             */
            SOPRANO_EXPORT QUrl depicts();

            /**
             * http://xmlns.com/foaf/0.1/dnaChecksum 
             * 
             * A checksum for the DNA of some thing. Joke. 
             */
            SOPRANO_EXPORT QUrl dnaChecksum();

            /**
             * http://xmlns.com/foaf/0.1/family_name 
             * 
             * The family_name of some person. 
             */
            SOPRANO_EXPORT QUrl family_name();

            /**
             * http://xmlns.com/foaf/0.1/firstName 
             * 
             * The first name of a person. 
             */
            SOPRANO_EXPORT QUrl firstName();

            /**
             * http://xmlns.com/foaf/0.1/fundedBy 
             * 
             * An organization funding a project or person. 
             */
            SOPRANO_EXPORT QUrl fundedBy();

            /**
             * http://xmlns.com/foaf/0.1/geekcode 
             * 
             * A textual geekcode for this person, see http://www.geekcode.com/geek.html 
             */
            SOPRANO_EXPORT QUrl geekcode();

            /**
             * http://xmlns.com/foaf/0.1/gender 
             * 
             * The gender of this Agent (typically but not necessarily 'male' 
             * or 'female'). 
             */
            SOPRANO_EXPORT QUrl gender();

            /**
             * http://xmlns.com/foaf/0.1/givenname 
             * 
             * The given name of some person. 
             */
            SOPRANO_EXPORT QUrl givenname();

            /**
             * http://xmlns.com/foaf/0.1/holdsAccount 
             * 
             * Indicates an account held by this agent. 
             */
            SOPRANO_EXPORT QUrl holdsAccount();

            /**
             * http://xmlns.com/foaf/0.1/homepage 
             * 
             * A homepage for some thing. 
             */
            SOPRANO_EXPORT QUrl homepage();

            /**
             * http://xmlns.com/foaf/0.1/icqChatID 
             * 
             * An ICQ chat ID 
             */
            SOPRANO_EXPORT QUrl icqChatID();

            /**
             * http://xmlns.com/foaf/0.1/img 
             * 
             * An image that can be used to represent some thing (ie. those depictions 
             * which are particularly representative of something, eg. one's 
             * photo on a homepage). 
             */
            SOPRANO_EXPORT QUrl img();

            /**
             * http://xmlns.com/foaf/0.1/interest 
             * 
             * A page about a topic of interest to this person. 
             */
            SOPRANO_EXPORT QUrl interest();

            /**
             * http://xmlns.com/foaf/0.1/isPrimaryTopicOf 
             * 
             * A document that this thing is the primary topic of. 
             */
            SOPRANO_EXPORT QUrl isPrimaryTopicOf();

            /**
             * http://xmlns.com/foaf/0.1/jabberID 
             * 
             * A jabber ID for something. 
             */
            SOPRANO_EXPORT QUrl jabberID();

            /**
             * http://xmlns.com/foaf/0.1/knows 
             * 
             * A person known by this person (indicating some level of reciprocated 
             * interaction between the parties). 
             */
            SOPRANO_EXPORT QUrl knows();

            /**
             * http://xmlns.com/foaf/0.1/logo 
             * 
             * A logo representing some thing. 
             */
            SOPRANO_EXPORT QUrl logo();

            /**
             * http://xmlns.com/foaf/0.1/made 
             * 
             * Something that was made by this agent. 
             */
            SOPRANO_EXPORT QUrl made();

            /**
             * http://xmlns.com/foaf/0.1/maker 
             * 
             * An agent that made this thing. 
             */
            SOPRANO_EXPORT QUrl maker();

            /**
             * http://xmlns.com/foaf/0.1/mbox 
             * 
             * A personal mailbox, ie. an Internet mailbox associated with 
             * exactly one owner, the first owner of this mailbox. This is a 
             * 'static inverse functional property', in that there is (across 
             * time and change) at most one individual that ever has any particular 
             * value for foaf:mbox. 
             */
            SOPRANO_EXPORT QUrl mbox();

            /**
             * http://xmlns.com/foaf/0.1/mbox_sha1sum 
             * 
             * The sha1sum of the URI of an Internet mailbox associated with 
             * exactly one owner, the first owner of the mailbox. 
             */
            SOPRANO_EXPORT QUrl mbox_sha1sum();

            /**
             * http://xmlns.com/foaf/0.1/member 
             * 
             * Indicates a member of a Group 
             */
            SOPRANO_EXPORT QUrl member();

            /**
             * http://xmlns.com/foaf/0.1/membershipClass 
             * 
             * Indicates the class of individuals that are a member of a Group 
             */
            SOPRANO_EXPORT QUrl membershipClass();

            /**
             * http://xmlns.com/foaf/0.1/msnChatID 
             * 
             * An MSN chat ID 
             */
            SOPRANO_EXPORT QUrl msnChatID();

            /**
             * http://xmlns.com/foaf/0.1/myersBriggs 
             * 
             * A Myers Briggs (MBTI) personality classification. 
             */
            SOPRANO_EXPORT QUrl myersBriggs();

            /**
             * http://xmlns.com/foaf/0.1/name 
             * 
             * A name for some thing. 
             */
            SOPRANO_EXPORT QUrl name();

            /**
             * http://xmlns.com/foaf/0.1/nick 
             * 
             * A short informal nickname characterising an agent (includes 
             * login identifiers, IRC and other chat nicknames). 
             */
            SOPRANO_EXPORT QUrl nick();

            /**
             * http://xmlns.com/foaf/0.1/openid 
             * 
             * An OpenID for an Agent. 
             */
            SOPRANO_EXPORT QUrl openid();

            /**
             * http://xmlns.com/foaf/0.1/page 
             * 
             * A page or document about this thing. 
             */
            SOPRANO_EXPORT QUrl page();

            /**
             * http://xmlns.com/foaf/0.1/pastProject 
             * 
             * A project this person has previously worked on. 
             */
            SOPRANO_EXPORT QUrl pastProject();

            /**
             * http://xmlns.com/foaf/0.1/phone 
             * 
             * A phone, specified using fully qualified tel: URI scheme (refs: 
             * http://www.w3.org/Addressing/schemes.html#tel). 
             */
            SOPRANO_EXPORT QUrl phone();

            /**
             * http://xmlns.com/foaf/0.1/plan 
             * 
             * A .plan comment, in the tradition of finger and '.plan' files. 
             */
            SOPRANO_EXPORT QUrl plan();

            /**
             * http://xmlns.com/foaf/0.1/primaryTopic 
             * 
             * The primary topic of some page or document. 
             */
            SOPRANO_EXPORT QUrl primaryTopic();

            /**
             * http://xmlns.com/foaf/0.1/publications 
             * 
             * A link to the publications of this person. 
             */
            SOPRANO_EXPORT QUrl publications();

            /**
             * http://xmlns.com/foaf/0.1/schoolHomepage 
             * 
             * A homepage of a school attended by the person. 
             */
            SOPRANO_EXPORT QUrl schoolHomepage();

            /**
             * http://xmlns.com/foaf/0.1/sha1 
             * 
             * A sha1sum hash, in hex. 
             */
            SOPRANO_EXPORT QUrl sha1();

            /**
             * http://xmlns.com/foaf/0.1/surname 
             * 
             * The surname of some person. 
             */
            SOPRANO_EXPORT QUrl surname();

            /**
             * http://xmlns.com/foaf/0.1/theme 
             * 
             * A theme. 
             */
            SOPRANO_EXPORT QUrl theme();

            /**
             * http://xmlns.com/foaf/0.1/thumbnail 
             * 
             * A derived thumbnail image. 
             */
            SOPRANO_EXPORT QUrl thumbnail();

            /**
             * http://xmlns.com/foaf/0.1/tipjar 
             * 
             * A tipjar document for this agent, describing means for payment 
             * and reward. 
             */
            SOPRANO_EXPORT QUrl tipjar();

            /**
             * http://xmlns.com/foaf/0.1/topic 
             * 
             * A topic of some page or document. 
             */
            SOPRANO_EXPORT QUrl topic();

            /**
             * http://xmlns.com/foaf/0.1/topic_interest 
             * 
             * A thing of interest to this person. 
             */
            SOPRANO_EXPORT QUrl topic_interest();

            /**
             * http://xmlns.com/foaf/0.1/weblog 
             * 
             * A weblog of some thing (whether person, group, company etc.). 
             */
            SOPRANO_EXPORT QUrl weblog();

            /**
             * http://xmlns.com/foaf/0.1/workInfoHomepage 
             * 
             * A work info homepage of some person; a page about their work for 
             * some organization. 
             */
            SOPRANO_EXPORT QUrl workInfoHomepage();

            /**
             * http://xmlns.com/foaf/0.1/workplaceHomepage 
             * 
             * A workplace homepage of some person; the homepage of an organization 
             * they work for. 
             */
            SOPRANO_EXPORT QUrl workplaceHomepage();

            /**
             * http://xmlns.com/foaf/0.1/yahooChatID 
             * 
             * A Yahoo chat ID 
             */
            SOPRANO_EXPORT QUrl yahooChatID();

            /**
             * http://xmlns.com/wordnet/1.6/Agent-3 
             */
            SOPRANO_EXPORT QUrl Agent_3();

            /**
             * http://xmlns.com/wot/0.1/assurance 
             */
            SOPRANO_EXPORT QUrl assurance();

            /**
             * http://xmlns.com/wot/0.1/src_assurance 
             */
            SOPRANO_EXPORT QUrl src_assurance();
        }
    }
}

#endif
