/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2008 Sebastian Trueg <trueg@kde.org>
 *               2000 Espen Sand, espen@kde.org
 *               2001-2002 Marc Mutz <mutz@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _IDENTITY_KCM_H_
#define _IDENTITY_KCM_H_

#include <KCModule>

namespace KPIMIdentities {
    class IdentityManager;
}
namespace KMail {
    class IdentityDialog;
    class IdentityListView;
    class IdentityListViewItem;
}
class QPushButton;
class QPoint;


class KCMIdentity : public KCModule
{
    Q_OBJECT

public:
    KCMIdentity( QWidget* parent, const QVariantList& );
    ~KCMIdentity();

    void load();
    void save();

    static KPIMIdentities::IdentityManager* identityManager();

private Q_SLOTS:
    void slotNewIdentity();
    void slotModifyIdentity();
    void slotRemoveIdentity();
    void slotRenameIdentity();
    void slotRenameIdentity( KMail::IdentityListViewItem *item , const QString &text );
    void slotSetAsDefault();
    void slotIdentitySelectionChanged();
    void slotContextMenu( KMail::IdentityListViewItem *item, const QPoint &pos );

private:
    void refreshList();

    KPIMIdentities::IdentityManager* m_identityManager;

    KMail::IdentityDialog   *mIdentityDialog;
    int                      mOldNumberOfIdentities;

    KMail::IdentityListView *mIdentityList;
    QPushButton             *mModifyButton;
    QPushButton             *mRenameButton;
    QPushButton             *mRemoveButton;
    QPushButton             *mSetAsDefaultButton;
};

#endif
