/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "scriptelement.h"
#include "../plug.h"
#include "../socket.h"

#include <Soprano/Model>
#include <Soprano/StatementIterator>
#include <Soprano/QueryResultIterator>
#include <Soprano/Parser>
#include <Soprano/Serializer>
#include <Soprano/Global>
#include <Soprano/RdfSerialization>
#include <Soprano/PluginManager>

#include <QtCore/QUrl>
#include <QtCore/QProcess>


K_EXPORT_PLASMA_APPLET(konduit_script, Konduit::Elements::ScriptElement)


using namespace Konduit::Elements;

ScriptElement::ScriptElement( QObject* parent, const QVariantList& args )
    : AbstractParseElement( parent, args ),
      Sink()
{
    addPlug( "input" );

    ConfigurationOption o( "script", QVariant::Url );
    o.setLabel( i18n( "Script" ) );
    addConfigurationOption( o );

    resize( QSizeF(250, 200) );
    setMinimumSize( QSizeF(200, 100) );

    createStandardGui();

    setToolTip("Script");
    
    emit configurationValueChanged( QString() );//trigger updateStandardGui
}


Soprano::StatementIterator ScriptElement::parseData( const Soprano::Parser* parser ) const
{
    kDebug() << "running the script" << endl;

    QString script = optionSetting("script").value().toUrl().toLocalFile();
    kDebug() << script << endl;

    // serialize the input and put it in a temp file
    Plug* p = plug( "input" );
    if ( p->socket() ) {

        const Soprano::Serializer* serializer = Soprano::PluginManager::instance()->discoverSerializerForSerialization( serialization(), userSerialization() );
        if ( !serializer ) {
            kDebug() << "Could not find serializer for" << userSerialization();
            return 0;
        }

        QString data;
        QTextStream dataStream( &data, QIODevice::WriteOnly );
        serializer->serialize( p->socket()->data(), dataStream, serialization(), userSerialization() );
        QProcess p;
        p.start( script );
        if ( p.waitForStarted() ) {
            p.write( data.toUtf8() );
            p.closeWriteChannel();
            p.waitForFinished();
            QString reply = QString::fromUtf8( p.readAllStandardOutput() );
            QTextStream replyStream( &reply );
            return parser->parseStream( replyStream, baseUri(), serialization(), userSerialization() );
        }
        else {
            kDebug() << "FIXME: Do error reporting";
            return 0;
        }
    }

    return Soprano::StatementIterator();
}

#include "scriptelement.moc"
