/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "sparqlsourceelement.h"
#include "socket.h"

#include <Soprano/Model>
#include <Soprano/QueryResultIterator>
#include <Soprano/StatementIterator>

#include <KDebug>

#include <plasma/widgets/lineedit.h>
#include <plasma/widgets/textedit.h>

#include <QObject>
#include <QGraphicsLinearLayout>
#include <KTextEdit>

using namespace Plasma;

class Konduit::Elements::SparqlSourceElement::Private
{
public:
    Private( SparqlSourceElement* parent )
        : q( parent ) {
    }

    Plasma::TextEdit* edit;

    void _k_configChanged();
    void _k_textChanged();

private:
    SparqlSourceElement* q;
};


void Konduit::Elements::SparqlSourceElement::Private::_k_configChanged()
{
    q->blockSignals( true );
    edit->nativeWidget()->setText( q->query() );
    q->blockSignals( false );
}


void Konduit::Elements::SparqlSourceElement::Private::_k_textChanged()
{
    q->blockSignals( true );
    q->setQuery( edit->nativeWidget()->toPlainText() );
    q->blockSignals( false );
}



Konduit::Elements::SparqlSourceElement::SparqlSourceElement( QGraphicsItem* parent, const QString& serviceId, uint appletId )
    : SourceElement( parent, serviceId, appletId ),
      d( new Private( this ) )
{
    initElement();
}


Konduit::Elements::SparqlSourceElement::SparqlSourceElement( QObject* parent, const QVariantList& args )
    : SourceElement( parent, args ),
      d( new Private( this ) )
{
    initElement();
    setToolTip("SPARQL Source");
    resize( QSizeF(400, 200) );
    setMinimumSize( QSizeF(160, 120) );
}


Konduit::Elements::SparqlSourceElement::~SparqlSourceElement()
{
    delete d;
}


void Konduit::Elements::SparqlSourceElement::initElement()
{
    addSocket( "querydata" );

    QGraphicsLinearLayout* lay = new QGraphicsLinearLayout( Qt::Vertical, this);

    ConfigurationOption o( "query", QVariant::String, "construct { ?r ?p ?o } where { ?r ?p ?o }" );
    o.setLabel( i18n( "Query" ) );
    addConfigurationOption( o );

    d->edit = new Plasma::TextEdit( this );
    lay->addItem( d->edit );
    d->edit->setText( query() );
    d->edit->nativeWidget()->setAcceptRichText( false );
    d->edit->nativeWidget()->setTextBackgroundColor( Qt::darkGray );

    connect( this, SIGNAL( configurationValueChanged( const QString& ) ), 
             this, SLOT( _k_configChanged() ) );
    connect( d->edit, SIGNAL( textChanged( ) ),
             this, SLOT( _k_textChanged() ) );
}


void Konduit::Elements::SparqlSourceElement::setQuery( const QString& query )
{
    setOption( "query", ConfigurationSetting( query ) );
}


QString Konduit::Elements::SparqlSourceElement::query() const
{
    return optionSetting( "query" ).value().toString();
}


Soprano::StatementIterator Konduit::Elements::SparqlSourceElement::data( const Socket* s ) const
{
    kDebug() << "performing query" << query();
    Q_UNUSED( s )
    return nepomukMainModel()->executeQuery( query(), Soprano::Query::QUERY_LANGUAGE_SPARQL ).iterateStatements();
}

/*
QSizeF Konduit::Elements::SparqlSourceElement::sizeHint( Qt::SizeHint, const QSizeF& ) const
{
    // some useful bigger size for a multiline query input
    return QSizeF( 400, 200 );
}
*/

#include "sparqlsourceelement.moc"

