/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "visualsparqlsourceelement.h"
#include "socket.h"
#include "visualquerybuilder.h"
//#include "visualquerybuilder.cpp"

#include <Soprano/Model>
#include <Soprano/QueryResultIterator>
#include <Soprano/StatementIterator>

#include <KDebug>

#include <plasma/widgets/lineedit.h>
#include <plasma/widgets/textedit.h>

#include <QObject>
#include <QGraphicsLinearLayout>
#include <KTextEdit>

using namespace Plasma;

class Konduit::Elements::VisualSparqlSourceElement::Private
{
public:
    Private( VisualSparqlSourceElement* parent )
        : q( parent ) {
    }

    Plasma::TextEdit* edit;

    void _k_configChanged();
    void _k_textChanged();

private:
    VisualSparqlSourceElement* q;
};


void Konduit::Elements::VisualSparqlSourceElement::Private::_k_configChanged()
{
    q->blockSignals( true );
    edit->nativeWidget()->setText( q->query() );
    q->blockSignals( false );
}


void Konduit::Elements::VisualSparqlSourceElement::Private::_k_textChanged()
{
    q->blockSignals( true );
    q->setQuery( edit->nativeWidget()->toPlainText() );
    q->blockSignals( false );
}



Konduit::Elements::VisualSparqlSourceElement::VisualSparqlSourceElement( QGraphicsItem* parent, const QString& serviceId, uint appletId )
    : SourceElement( parent, serviceId, appletId ),
      d( new Private( this ) )
{
    initElement();
}


Konduit::Elements::VisualSparqlSourceElement::VisualSparqlSourceElement( QObject* parent, const QVariantList& args )
    : SourceElement( parent, args ),
      d( new Private( this ) )
{
    initElement();
}


Konduit::Elements::VisualSparqlSourceElement::~VisualSparqlSourceElement()
{
    delete d;
}


void Konduit::Elements::VisualSparqlSourceElement::initElement()
{
    /*
    ConfigurationOption option( "output", QVariant::Url );
    option.setIsListOption( true );
    option.setLabel( i18n( "Output" ) );
    addConfigurationOption( option );

    ConfigurationOption option1( "conditions", QVariant::Url );
    option1.setIsListOption( true );
    option1.setLabel( i18n( "Conditions" ) );
    addConfigurationOption( option1 );
    */
    
    ConfigurationOption o( "query", QVariant::String, "CONSTRUCT { ?s ?p ?o } WHERE { ?s ?p ?o }" );
    o.setLabel( i18n( "Query" ) );
    addConfigurationOption( o );

    addSocket( "querydata" );

    QGraphicsLinearLayout* lay = new QGraphicsLinearLayout( Qt::Vertical, this);
    d->edit = new Plasma::TextEdit( this );
    lay->addItem( d->edit );
    d->edit->setText( query() );
    d->edit->nativeWidget()->setAcceptRichText( false );
    d->edit->nativeWidget()->setReadOnly( true );

    connect( this, SIGNAL( configurationValueChanged( const QString& ) ),
             this, SLOT( _k_configChanged() ) );
    connect( d->edit, SIGNAL( textChanged( ) ),
             this, SLOT( _k_textChanged() ) );

    setToolTip("Visual SPARQL Source");
    resize( QSizeF(400, 200) );
    setMinimumSize( QSizeF(160, 120) );

    setHasConfigurationInterface( true );
}


void Konduit::Elements::VisualSparqlSourceElement::showKonduitConfigurationInterface()
{
    VisualQueryBuilder* vqb = new VisualQueryBuilder( 0, 0,  query() );
    connect( vqb, SIGNAL( queryChanged( QString ) ),
             this, SLOT( setQuery( QString ) ) );
    vqb->show();
}


void Konduit::Elements::VisualSparqlSourceElement::setQuery( QString query )
{
    setOption( "query", ConfigurationSetting( query ) );
}


QString Konduit::Elements::VisualSparqlSourceElement::query() const
{
    return optionSetting( "query" ).value().toString();
}


Soprano::StatementIterator Konduit::Elements::VisualSparqlSourceElement::data( const Socket* s ) const
{
    kDebug() << "performing query" << query();
    Q_UNUSED( s )
    return nepomukMainModel()->executeQuery( query(), Soprano::Query::QUERY_LANGUAGE_SPARQL ).iterateStatements();
}

/*
QSizeF Konduit::Elements::VisualSparqlSourceElement::sizeHint( Qt::SizeHint, const QSizeF& ) const
{
    // some useful bigger size for a multiline query input
    return QSizeF( 400, 200 );
}
*/

#include "visualsparqlsourceelement.moc"

