/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _KONDUIT_VISUAL_SPARQL_SOURCEELEMENT_H_
#define _KONDUIT_VISUAL_SPARQL_SOURCEELEMENT_H_

#include "sourceelement.h"

#include "konduit_export.h"

#include <QtCore/QString>

namespace Konduit {

    class Socket;

    namespace Elements {

        /**
         * Simple SourceElement that gets data from the Nepomuk
         * data repository through a user defined SPARQL query.
         */
        class KONDUIT_EXPORT VisualSparqlSourceElement : public SourceElement
        {
            Q_OBJECT

        public:
            VisualSparqlSourceElement( QGraphicsItem* parent, const QString& serviceId, uint appletId );
            VisualSparqlSourceElement( QObject* parent, const QVariantList& args );
            virtual ~VisualSparqlSourceElement();

            /**
             * \param query Needs to be a graph query.
             */
            //void setQuery( const QString& query );
            QString query() const;

            Soprano::StatementIterator data( const Socket* ) const;
            //void showConfigurationInterface();

        public Q_SLOTS:
            void showKonduitConfigurationInterface();
            void setQuery( QString query );

        private:
            //QSizeF sizeHint( Qt::SizeHint which, const QSizeF& constraint ) const;
            void initElement();

            class Private;
            Private* const d;

            Q_PRIVATE_SLOT( d, void _k_configChanged() )
            Q_PRIVATE_SLOT( d, void _k_textChanged() )
        };
    }
}

K_EXPORT_PLASMA_APPLET(konduit_visualsparqlsource, Konduit::Elements::VisualSparqlSourceElement)

#endif
