/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "queryelement.h"
#include "plug.h"
#include "socket.h"

#include <Soprano/StorageModel>
#include <Soprano/StatementIterator>
#include <Soprano/Backend>
#include <Soprano/Global>
#include <Soprano/PluginManager>
#include <Soprano/QueryResultIterator>


class Konduit::Elements::QueryElement::Private
{
public:
};

Konduit::Elements::QueryElement::~QueryElement()
{
    delete d;
}


Konduit::Elements::QueryElement::QueryElement( QObject* parent, const QVariantList& args )
    : MemoryModelElement( parent, args ),
      d( new Private() )
{
    ConfigurationOption o( "query", QVariant::String );
    o.setLabel( i18n( "Query" ) );
    addConfigurationOption( o );

    ConfigurationOption op( "queryLanguage", QVariant::String, Soprano::Query::queryLanguageToString( Soprano::Query::QueryLanguageSparql ) );
    op.setLabel( i18n( "Query language" ) );
    op.setIsListOption( true );
    op.setForceSuggestedValue( true );
    op.addSuggestedValue( Soprano::Query::queryLanguageToString( Soprano::Query::QueryLanguageSparql ) );
    op.addSuggestedValue( Soprano::Query::queryLanguageToString( Soprano::Query::QueryLanguageRdql ) );
    addConfigurationOption( op );
}


void Konduit::Elements::QueryElement::setQuery( const QString& query,
                                                Soprano::Query::QueryLanguage language,
                                                const QString& userQueryLanguage )
{
    setOption( "query", ConfigurationSetting( query ) );
    setOption( "queryLanguage", ConfigurationSetting( Soprano::Query::queryLanguageToString( language, userQueryLanguage ) ) );
}


QString Konduit::Elements::QueryElement::query() const
{
    return optionSetting( "query" ).value().toString();
}


Soprano::Query::QueryLanguage Konduit::Elements::QueryElement::queryLanguage() const
{
    return Soprano::Query::queryLanguageFromString( optionSetting( "queryLanguage" ).value().toString() );
}


QString Konduit::Elements::QueryElement::userQueryLanaguage() const
{
    if ( queryLanguage() == Soprano::Query::QueryLanguageUser )
        return optionSetting( "queryLanguage" ).value().toString();
    else
        return QString();
}


Soprano::QueryResultIterator Konduit::Elements::QueryElement::queryResult() const
{
    // FIXME: memory leak as we do not remember the Model
    Soprano::Model* m = model();
    if ( m ) {
        return m->executeQuery( query(), queryLanguage(), userQueryLanaguage() );
    }
    else {
        return Soprano::QueryResultIterator();
    }
}

#include "queryelement.moc"
