/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _KONDUIT_QUERY_ELEMENT_H_
#define _KONDUIT_QUERY_ELEMENT_H_

#include "memorymodelelement.h"
#include "konduit_export.h"

#include <Soprano/Query/QueryLanguage>
#include <Soprano/QueryResultIterator>

namespace Konduit {
    namespace Elements {
        /**
         * Simple base element that performs a query on the data
         * from its inputs and disaplays the result.
         *
         * Usage: Derive from QueryElement, add one or more plugs,
         * add sockets (in this case the new element needs to be derived from
         * Source, too) or gui elements to start the query. Then call
         * queryResult() from the data() method or from GUI slots.
         */
        class KONDUIT_EXPORT QueryElement : public MemoryModelElement
        {
            Q_OBJECT

        public:
            QueryElement( QObject* parent, const QVariantList& args );
            virtual ~QueryElement();

            /**
             * Set the query.
             * \sa Soprano::Model::executeQuery
             */
            void setQuery( const QString& query, Soprano::Query::QueryLanguage language, const QString& userQueryLanguage = QString() );

            QString query() const;

            Soprano::Query::QueryLanguage queryLanguage() const;

            /**
             * \sa Soprano::Query::QueryLanguageUser
             */
            QString userQueryLanaguage() const;

            Soprano::QueryResultIterator queryResult() const;

        private:
            class Private;
            Private* const d;
        };
    }
}

#endif
