/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "querythread.h"
#include "visualquerybuilder.h"

#include <QTime>

#include <KDebug>


void Konduit::QueryThread::run()
{

    QList<QString> res = runQuery( query, freeVar );
    //kDebug() << "\n\n Results: for " << query << ": \n" << res;
    emit queryDone( res );
}

void Konduit::QueryThread::setQuery( QString _query, QString _freeVar ) 
{
    query = _query;
    freeVar = _freeVar;
}

QList<QString> Konduit::QueryThread::runQuery( QString query, QString freeVar )
{
    QString s = "SELECT DISTINCT ?" + freeVar + " WHERE { " + query + " } LIMIT 50";//"SELECT * WHERE {" + query + "} LIMIT 1";
    //add prefixes
    s = VisualQueryBuilderConsts::addPrefixes( s );
    //kDebug() << "--- Running query: " << s;

    Soprano::Model* m = nepomukMainModel();
    Soprano::QueryResultIterator it = m->executeQuery( s,
                                                         Soprano::Query::QueryLanguageSparql );
    QList<QString> res;
    QList<Soprano::BindingSet> allStatements = it.allBindings();
 
    QString val;
    foreach (Soprano::BindingSet s, allStatements ) {
              Soprano::Node n = s.value(freeVar);
              if ( n.isResource() ) {
                  val = n.uri().toString();
                  val = VisualQueryBuilderConsts::getPrefixForm( val );
              } else if ( n.isLiteral() ) {
                  val = "\"" + n.literal().toString() + "\"^^<"+ n.literal().dataTypeUri().toString()+">";
              }
              //kDebug() << "--- Found: " << val;
              res << val;
    }
    
    //kDebug() << "--- *** The results: " << res;
    
    return res;
}

int Konduit::QueryThread::countQueryResults( QString query )
{
    QString s = VisualQueryBuilderConsts::addPrefixes( query );
//    kDebug() << "*** Counting results for: " << s;

    Soprano::Model* m = nepomukMainModel();
    Soprano::QueryResultIterator it = m->executeQuery( s,
                                                         Soprano::Query::QueryLanguageSparql );

    QList<Soprano::BindingSet> allStatements = it.allBindings();//Elements();    
    
    int resNo = allStatements.count();;
//    kDebug() << "*** Counted " << resNo << " results";
    
    return resNo;
}


static Soprano::Model* s_model = 0;

Soprano::Model* Konduit::QueryThread::nepomukMainModel()
{
    // we use a dummy test model here
    if ( !s_model ) {
        static Soprano::Client::DBusClient client( "org.kde.NepomukServer" );
        s_model = client.createModel( "main" );
    }

    if ( !s_model ) {
        s_model = new Soprano::Util::DummyModel();
    }

    return s_model;
}

#include "querythread.moc"
