/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef QUERYTHREAD_H
#define QUERYTHREAD_H

#include <QThread>

#include <Soprano/Soprano>
#include <Soprano/Model>
#include <Soprano/Client/DBusClient>
#include <Soprano/Client/DBusModel>
#include <Soprano/Util/DummyModel>

#include "visualquerybuilderconsts.h"

namespace Konduit {

/*! Thread class that will execute queries (and perform some utility functions, like counting query results, stc. ) 
*/

class QueryThread : public QThread
{

Q_OBJECT

public:

    /** Runs thread
    */
    void run();
    
    /** Sets the query to be run on the RDF repository
    */
    void setQuery( QString query, QString freeVar );
    
    /** SYNCHRONOUS function.
       Returns query results of a simple SELECT query - utility method
       query - the query to run
       freeVar - the free variable to get bindings for
    */
    QList<QString> runQuery( QString query, QString freeVar );
    
    /** Counts the query results of a simple SELECT query - utility method
    */
    int countQueryResults( QString query );
        
    /** Convenience method to access the main Nepomuk data model.
    */
    Soprano::Model* nepomukMainModel();

Q_SIGNALS:
    void queryDone( QStringList );
    
private:
    
    QString query;
    QString freeVar;
}; 

}

#endif
