/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _SIBLING_GRAPHICS_ITEM_H_
#define _SIBLING_GRAPHICS_ITEM_H_

/**
 * A sibling item is comparable to a child item that lives
 * outside of the parent. It is thus inserted into the scene along
 * with its parent and moved when the parent is moved.
 */
class SiblingGraphicsItem : public QGraphicsItem
{
public:
    /**
     * \param sibling This is the sibling parent. Do not confuse this with
     * QGraphicsItem::parentItem()
     */
    SiblingGraphicsItem( QGraphicsItem* sibling = 0 );
    virtual ~SiblingGraphicsItem();

    /**
     * Set the sibling parent.
     * \param sibling The parent item. The SiblingGraphicsItem will 
     * become a child of sibling's parent and will be moved along
     * with it.
     */ 
    void setSiblingParent( QGraphicsItem* sibling );
    QGraphicsItem* siblingParent() const;

protected:
    /**
     * Makes sure the sibling is moved along with its parent sibling.
     * Implementations need to call this method.
     */
    virtual bool sceneEventFilter( QGraphicsItem* watched, QEvent* event );

private:
    class Private;
    Private* const d;
};

#endif
