/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "simpledatasource.h"
#include "socket.h"

#include <Soprano/Util/SimpleStatementIterator>


class Konduit::Elements::SimpleDataSource::Private
{
public:
    QList<Soprano::Statement> statements;
};


Konduit::Elements::SimpleDataSource::SimpleDataSource( QGraphicsItem* parent, const QString& serviceId, uint appletId, const QList<Soprano::Statement>& statements )
    : SourceElement( parent, serviceId, appletId ),
      d( new Private() )
{
    d->statements = statements;
    addSocket( "data" );
}


Konduit::Elements::SimpleDataSource::SimpleDataSource( QObject* parent, const QVariantList& args )
    : SourceElement( parent, args ),
      d( new Private() )
{
    addSocket( "data" );
}


Konduit::Elements::SimpleDataSource::~SimpleDataSource()
{
    delete d;
}


void Konduit::Elements::SimpleDataSource::setData( const QList<Soprano::Statement>& statements )
{
    d->statements = statements;
}


QList<Soprano::Statement> Konduit::Elements::SimpleDataSource::data() const
{
    return d->statements;
}


Soprano::StatementIterator Konduit::Elements::SimpleDataSource::data( const Socket* s ) const
{
    Q_UNUSED( s );
    return Soprano::Util::SimpleStatementIterator( d->statements );
}
