/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "sink.h"
#include "plug.h"
#include "element.h"

#include <QtCore/QMap>
#include <QtCore/QPointer>


class Konduit::Sink::Private
{
public:
    // We use QPointer since the plugs may have already been deleted
    // by the QGraphicsScene
    // we cannot use QMap since we want the plugs to be sorted by insertion time
    QList<QPointer<Plug> > plugs;

    Plug* findPlug( const QString& name ) {
        for ( int i = 0; i < plugs.count(); ++i ) {
            if ( plugs[i]->name() == name ) {
                return plugs[i];
            }
        }
        return 0;
    }
};


Konduit::Sink::Sink()
    : d( new Private() )
{
}


Konduit::Sink::~Sink()
{
    qDeleteAll( d->plugs );
    delete d;
}


int Konduit::Sink::countPlugs() const
{
    return d->plugs.count();
}


QStringList Konduit::Sink::allPlugNames() const
{
    QStringList names;
    foreach( QPointer<Plug> plug, d->plugs ) {
        names << plug->name();
    }
    return names;
}


QList<Konduit::Plug*> Konduit::Sink::allPlugs() const
{
    QList<Konduit::Plug*> l;
    Q_FOREACH( QPointer<Plug> p, d->plugs ) {
        l.append( p );
    }
    return l;
}


Konduit::Plug* Konduit::Sink::plug( const QString& name ) const
{
    if ( !name.isEmpty() ) {
        return d->findPlug( name );
    }
    else if ( countPlugs() == 1 ) {
        return d->plugs.first();
    }
    return 0;
}


Konduit::Plug* Konduit::Sink::addPlug( const QString& name )
{
    Q_ASSERT( !d->findPlug( name ) );

    // FIXME: possible memleak
    Plug* p = new Plug( this, name );
    d->plugs.append( p );
    p->connect( p,
                SIGNAL( plugStateChanged( Konduit::Socket*, Konduit::Socket* ) ),
                dynamic_cast<Element*>( this ),
                SLOT( slotPlugStateChanged( Konduit::Socket*, Konduit::Socket* ) ) );
    return p;
}
