/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _KONDUIT_SINK_H_
#define _KONDUIT_SINK_H_

#include <QtCore/QStringList>

#include "konduit_export.h"

namespace Konduit {

    class Plug;

    /**
     * Interface for all data sink elements.
     */
    class KONDUIT_EXPORT Sink
    {
    public:
        virtual ~Sink();

        int countPlugs() const;

        QList<Plug*> allPlugs() const;
        QStringList allPlugNames() const;

        /**
         * Get a plug that this source provides.
         *
         * \param name The name of the plug in question. If empty
         * and the source has only a single plug, that is returned.
         *
         * \return The requested Plug or 0 if no plug of that name exists.
         */
        Plug* plug( const QString& name = QString() ) const;

    protected:
        Sink();

        Plug* addPlug( const QString& name );

    private:
        class Private;
        Private* const d;
    };
}

#endif
