/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _KONDUIT_SOCKET_H_
#define _KONDUIT_SOCKET_H_

#include <QtCore/QString>
#include <QtCore/QObject>
#include <QtGui/QGraphicsItem>

#include "konduit_export.h"

namespace Soprano {
    class StatementIterator;
}

namespace Konduit {

    class Sink;
    class Source;
    class Plug;

    /**
     * A Sink can have multiple sockets. Each Socket
     * can be connected to a Plug.
     */
    class KONDUIT_EXPORT Socket : public QObject, public QGraphicsItem
    {
        Q_OBJECT

    public:
        ~Socket();

        /**
         * The name of the socket.
         */
        QString name() const;

        /**
         * The Source this socket belongs to.
         */
        Source* source() const;

        /**
         * Plug the plug into thi socket.
         * The parent Source will read data from the Plug.
         */
        bool plugIn( Plug* source );

        void unplug();

        Plug* plug() const;

        QRectF boundingRect() const;
        void paint( QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget = 0 );

        /**
         * Retrive the stream of data from the source.
         * 
         * \sa Source::data
         */
        Soprano::StatementIterator data() const;

    Q_SIGNALS:
        void plugStateChanged( Konduit::Plug* oldPlug, Konduit::Plug* newPlug );

    private:
        Socket( Source* source, const QString& name = QString() );

        QVariant itemChange( GraphicsItemChange change, const QVariant& value );

        class Private;
        Private* const d;

        // Socket instances are owned by their parent Source
        friend class Source;
    };
}

#endif
