/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _KONDUIT_SOURCE_H_
#define _KONDUIT_SOURCE_H_

#include <QtCore/QStringList>

#include "konduit_export.h"

namespace Soprano {
    class StatementIterator;
}

namespace Konduit {

    class Socket;

    /**
     * Interface for all data source elements.
     */
    class KONDUIT_EXPORT Source
    {
    public:
        virtual ~Source();

        int countSockets() const;

        QList<Socket*> allSockets() const;
        QStringList allSocketNames() const;

        /**
         * Get a socket that this source provides.
         *
         * \param name The name of the socket in question. If empty
         * and the source has only a single socket, that is returned.
         *
         * \return The requested Socket or 0 if no socket of that name exists.
         */
        Socket* socket( const QString& name = QString() ) const;

        /**
         * Retrive the stream of data from the source.
         *
         * \param socket The socket from which the data is read.
         *
         * \sa Socket::data
         */
        virtual Soprano::StatementIterator data( const Socket* socket ) const = 0;

    protected:
        Source();

        Socket* addSocket( const QString& name );

        Socket* createSocket( Source* source, const QString& name );

    private:
        class Private;
        Private* const d;
    };
}

#endif
