/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "visualquerybuilder.h"
#include "visualquerybuilderconsts.h"


#include <Soprano/StatementIterator>
#include <Soprano/QueryResultIterator>
#include <Soprano/Query/QueryLanguage>
#include <Soprano/Node>

#include <KDebug>
#include <KLocale>

#include <QTime>


Konduit::VisualQueryBuilder::VisualQueryBuilder(QWidget* parent, Qt::WindowFlags fl, QString initialQuery)
    : QWidget(parent, fl)
{
    init();
    setupUi();
    processInitialQuery( initialQuery );
}



Konduit::VisualQueryBuilder::~VisualQueryBuilder()
{
}

/** *********************** INITIALIZATION FUNCTIONS **************************** 
*/

void Konduit::VisualQueryBuilder::init()
{
    //initialize QueryThread object for the SPARQL queries
    //qTh = new QueryThread();
    //initialize prefixes of static class
    VisualQueryBuilderConsts::prefixes();
    //build predicate list (in prefix form)
    if(predicates.empty()) {
        predicates.append( "?p" );
        predicates << qTh->runQuery("?s ?p ?o", "p") ;
    }
    
    //build types list
    typesC << VisualQueryBuilderConsts::xmlSchemaTypes();
    typesO << VisualQueryBuilderConsts::xmlSchemaTypes();
}


void Konduit::VisualQueryBuilder::setupUi()
{
    VisualQueryBuilder *visualquerybuilder = this;

    //Initialize UI components
    visualquerybuilder->resize(600, 450);
    buttonOK = new QPushButton(visualquerybuilder);
    buttonOK->setObjectName(QString::fromUtf8("buttonOK"));
    buttonOK->setGeometry(QRect(310, 410, 80, 25));

    buttonApply = new QPushButton(visualquerybuilder);
    buttonApply->setObjectName(QString::fromUtf8("buttonCancel_2"));
    buttonApply->setGeometry(QRect(410, 410, 80, 25));

    buttonCancel = new QPushButton(visualquerybuilder);
    buttonCancel->setObjectName(QString::fromUtf8("buttonCancel"));
    buttonCancel->setGeometry(QRect(510, 410, 80, 25));

    groupBoxC = new QGroupBox(visualquerybuilder);
    groupBoxC->setObjectName(QString::fromUtf8("groupBoxC"));
    groupBoxC->setGeometry(QRect(10, 210, 590, 190));

    textLabel1_2_4 = new QLabel(groupBoxC);
    textLabel1_2_4->setObjectName(QString::fromUtf8("textLabel1_2_4"));
    textLabel1_2_4->setGeometry(QRect(10, 70, 71, 20));
    textLabel1_2_4->setWordWrap(false);
    textLabel1_2_2_3 = new QLabel(groupBoxC);
    textLabel1_2_2_3->setObjectName(QString::fromUtf8("textLabel1_2_2_3"));
    textLabel1_2_2_3->setGeometry(QRect(10, 110, 71, 20));
    textLabel1_2_2_3->setWordWrap(false);
    textLabel1_4 = new QLabel(groupBoxC);
    textLabel1_4->setObjectName(QString::fromUtf8("textLabel1_4"));
    textLabel1_4->setGeometry(QRect(10, 30, 71, 20));
    textLabel1_4->setWordWrap(false);

    lineEditCS = new QLineEdit(groupBoxC);
    lineEditCS->setObjectName(QString::fromUtf8("lineEditCS"));
    lineEditCS->setGeometry(QRect(80, 30, 221, 21));
    lineEditCS->setText("?s");

    comboBoxCP = new ComboBox(predicates, groupBoxC);
    comboBoxCP->setObjectName(QString::fromUtf8("comboBoxCP"));
    comboBoxCP->setGeometry(QRect(80, 70, 221, 21));
    comboBoxCP->setEditable( true );
    comboBoxCP->insertItems( -1, predicates );
    comboBoxCP->setCurrentIndex( 0 );

    lineEditCO = new QLineEdit(groupBoxC);
    lineEditCO->setObjectName(QString::fromUtf8("lineEditCO"));
    lineEditCO->setGeometry(QRect(80, 110, 151, 21));
    lineEditCO->setText("?o");

    comboBoxTypesC = new QComboBox(groupBoxC);
    comboBoxTypesC->setGeometry(QRect(240, 110, 60, 25));
    comboBoxTypesC->setEditable( false );
    QFont font;
    font.setPointSize(6);
    comboBoxTypesC->setFont(font);
    //refreshTypes();//called later

    pushButtonAddC = new QPushButton(groupBoxC);
    pushButtonAddC->setObjectName(QString::fromUtf8("pushButtonAddC"));
    pushButtonAddC->setGeometry(QRect(170, 150, 61, 25));

    pushButtonRemoveC = new QPushButton(groupBoxC);
    pushButtonRemoveC->setObjectName(QString::fromUtf8("pushButtonRemoveC"));
    pushButtonRemoveC->setGeometry(QRect(240, 150, 61, 25));

    listWidgetC = new QListWidget(groupBoxC);
    listWidgetC->setGeometry(QRect(320, 30, 251, 151));

    groupBoxO = new QGroupBox(visualquerybuilder);
    groupBoxO->setObjectName(QString::fromUtf8("groupBoxO"));
    groupBoxO->setGeometry(QRect(10, 20, 590, 190));

    textLabel1_2 = new QLabel(groupBoxO);
    textLabel1_2->setObjectName(QString::fromUtf8("textLabel1_2"));
    textLabel1_2->setGeometry(QRect(10, 70, 71, 20));
    textLabel1_2->setWordWrap(false);
    textLabel1_2_2 = new QLabel(groupBoxO);
    textLabel1_2_2->setObjectName(QString::fromUtf8("textLabel1_2_2"));
    textLabel1_2_2->setGeometry(QRect(10, 110, 71, 20));
    textLabel1_2_2->setWordWrap(false);
    textLabel1 = new QLabel(groupBoxO);
    textLabel1->setObjectName(QString::fromUtf8("textLabel1"));
    textLabel1->setGeometry(QRect(10, 30, 71, 20));
    textLabel1->setWordWrap(false);

    comboBoxOP = new ComboBox(predicates, groupBoxO);
    comboBoxOP->setObjectName(QString::fromUtf8("comboBoxOP"));
    comboBoxOP->setGeometry(QRect(80, 70, 221, 21));
    comboBoxOP->setEditable( true );
    comboBoxOP->insertItems( -1, predicates );
    comboBoxOP->setCurrentIndex( 0 );

    lineEditOO = new QLineEdit(groupBoxO);
    lineEditOO->setObjectName(QString::fromUtf8("lineEditOO"));
    lineEditOO->setGeometry(QRect(80, 110, 151, 21));
    lineEditOO->setText("?o");

    comboBoxTypesO = new QComboBox(groupBoxO);
    comboBoxTypesO->setGeometry(QRect(240, 110, 60, 25));
    comboBoxTypesO->setEditable( false );
    font.setPointSize(6);
    comboBoxTypesO->setFont(font);
    //refreshTypes();
    
    lineEditOS = new QLineEdit(groupBoxO);
    lineEditOS->setObjectName(QString::fromUtf8("lineEditOS"));
    lineEditOS->setGeometry(QRect(80, 30, 221, 21));
    lineEditOS->setText("?s");

    pushButtonAddO = new QPushButton(groupBoxO);
    pushButtonAddO->setObjectName(QString::fromUtf8("pushButtonAddO"));
    pushButtonAddO->setGeometry(QRect(170, 140, 61, 25));
    //kDebug() << "Initialized types:" << VisualQueryBuilderConsts::xmlSchemaTypes();
    pushButtonRemoveO = new QPushButton(groupBoxO);
    pushButtonRemoveO->setObjectName(QString::fromUtf8("pushButtonRemoveO"));
    pushButtonRemoveO->setGeometry(QRect(240, 140, 61, 25));

    listWidgetO = new QListWidget(groupBoxO);
    listWidgetO->setGeometry(QRect(320, 30, 251, 151));

    //assign names and connections 
    retranslateUi();
    QObject::connect(pushButtonAddC, SIGNAL(clicked()), visualquerybuilder, SLOT(buttonAddC_clicked()));
    QObject::connect(pushButtonAddO, SIGNAL(clicked()), visualquerybuilder, SLOT(buttonAddO_clicked()));
    QObject::connect(pushButtonRemoveC, SIGNAL(clicked()), visualquerybuilder, SLOT(buttonRemoveC_clicked()));
    QObject::connect(pushButtonRemoveO, SIGNAL(clicked()), visualquerybuilder, SLOT(buttonRemoveO_clicked()));
    QObject::connect(buttonOK, SIGNAL(clicked()), visualquerybuilder, SLOT(buttonOK_clicked()));
    QObject::connect(buttonApply, SIGNAL(clicked()), visualquerybuilder, SLOT(buttonApply_clicked()));
    QObject::connect(buttonCancel, SIGNAL(clicked()), visualquerybuilder, SLOT(buttonCancel_clicked()));
    QObject::connect(pushButtonAddC, SIGNAL(clicked()), visualquerybuilder, SLOT(varsChanged()));
    
    QObject::connect(pushButtonAddC, SIGNAL(clicked()), visualquerybuilder, SLOT(varsChanged()));
    QObject::connect(pushButtonAddO, SIGNAL(clicked()), visualquerybuilder, SLOT(varsChanged()));
    
    QObject::connect(lineEditCO, SIGNAL(editingFinished()), visualquerybuilder, SLOT(textChangedCO()));
    QObject::connect(lineEditCS, SIGNAL(editingFinished()), visualquerybuilder, SLOT(editingFinishedC()));
    QObject::connect(lineEditCO, SIGNAL(editingFinished()), visualquerybuilder, SLOT(editingFinishedC()));
    QObject::connect(comboBoxCP->lineEdit(), SIGNAL(editingFinished()), visualquerybuilder, SLOT(editingFinishedC()));
    QObject::connect(comboBoxCP, SIGNAL(currentIndexChanged(int)), visualquerybuilder, SLOT(editingFinishedC()));
    QObject::connect(comboBoxTypesC, SIGNAL(currentIndexChanged(int)), visualquerybuilder, SLOT(editingFinishedC()));
    
    QObject::connect(lineEditOO, SIGNAL(editingFinished()), visualquerybuilder, SLOT(textChangedOO()));
    QObject::connect(lineEditOS, SIGNAL(editingFinished()), visualquerybuilder, SLOT(editingFinishedO()));
    QObject::connect(lineEditOO, SIGNAL(editingFinished()), visualquerybuilder, SLOT(editingFinishedO()));
    QObject::connect(comboBoxOP->lineEdit(), SIGNAL(editingFinished()), visualquerybuilder, SLOT(editingFinishedO()));
    QObject::connect(comboBoxOP, SIGNAL(currentIndexChanged(int)), visualquerybuilder, SLOT(editingFinishedO()));
    QObject::connect(comboBoxTypesO, SIGNAL(currentIndexChanged(int)), visualquerybuilder, SLOT(editingFinishedO()));

    textChangedCO();
    textChangedOO();

} // setupUi


void Konduit::VisualQueryBuilder::retranslateUi()
{
    VisualQueryBuilder *visualquerybuilder = this;

    visualquerybuilder->setWindowTitle(QApplication::translate("visualquerybuilder", "Visual SPARQL Query Builder", 0, QApplication::UnicodeUTF8));
    buttonOK->setText(i18n( "O&K" ) );
    buttonOK->setShortcut( i18n( "Alt+K" ) );
    buttonCancel->setText( i18n( "Ca&ncel" ) );
    buttonCancel->setShortcut( i18n( "Alt+N" ) );
    groupBoxC->setTitle(i18n( "Conditions" ) );
    textLabel1_2_4->setText( i18n( "Predicate:" ) );
    textLabel1_2_2_3->setText( i18n( "Object:" ) );
    textLabel1_4->setText( i18n( "Subject:" ) );
    pushButtonAddC->setText( i18n( "&Add" ) );
    pushButtonRemoveC->setText( i18n( "&Remove" ) );
    groupBoxO->setTitle( i18n( "Output" ) );
    textLabel1_2->setText( i18n( "Predicate:" ) );
    textLabel1_2_2->setText( i18n( "Object:" ) );
    textLabel1->setText( i18n( "Subject:" ) );
    pushButtonAddO->setText( i18n( "A&dd" ) );
    pushButtonRemoveO->setText( i18n( "R&emove" ) );
    buttonApply->setText( i18n( "A&pply" ) );
    buttonApply->setShortcut( i18n( "Alt+N" ) );

    Q_UNUSED(visualquerybuilder);
} // retranslateUi


void Konduit::VisualQueryBuilder::processInitialQuery( QString initialQuery )
{
    QRegExp rx(VisualQueryBuilderConsts::queryRegExp());

    int pos = rx.indexIn(initialQuery);
    if (pos > -1) {
        QString output = rx.cap(3);
        QString conditions = rx.cap(7);
        //kDebug() << "Output: " << output << ", Conditions: " << conditions;
        
        pos = 0;
        QRegExp rx1(VisualQueryBuilderConsts::tripleRegExp());
        while ((pos = rx1.indexIn( output, pos )) != -1) {
            QString tempRes = rx1.cap(1);
            new QListWidgetItem( tempRes, listWidgetO );
            //kDebug() << "Found output: " << tempRes;
            pos += rx1.matchedLength();
        }
        
        pos = 0;
        QRegExp rx2(VisualQueryBuilderConsts::tripleRegExp());
        while ((pos = rx2.indexIn( conditions, pos )) != -1) {
            QString tempRes = rx2.cap(1);
            new QListWidgetItem( tempRes, listWidgetC );
            //kDebug() << "Found condition: " << tempRes;
            pos += rx2.matchedLength();
        }
    }

}

/** **************** PUBLIC SLOTS *************************
*/


void Konduit::VisualQueryBuilder::buttonOK_clicked()
{
    emit queryChanged( constructQuery() );
    close();
}


void Konduit::VisualQueryBuilder::buttonApply_clicked()
{
    
    emit queryChanged( constructQuery() );
}


void Konduit::VisualQueryBuilder::buttonCancel_clicked()
{
    close();
}
    
        
void Konduit::VisualQueryBuilder::buttonAddO_clicked()
{
    QString object = VisualQueryBuilderConsts::getCanonicalForm( lineEditOO->text(), comboBoxTypesO->currentText() );
        
    if(!lineEditOS->text().isEmpty() && !comboBoxOP->currentText().isEmpty() && !lineEditOO->text().isEmpty()) {
        new QListWidgetItem( lineEditOS->text() + " " + comboBoxOP->currentText() + " " + object,
                             listWidgetO );
    }
}
    

void Konduit::VisualQueryBuilder::buttonAddC_clicked()
{
    QString object = VisualQueryBuilderConsts::getCanonicalForm( lineEditCO->text(), comboBoxTypesC->currentText() );
    
    if(!lineEditCS->text().isEmpty() && !comboBoxCP->currentText().isEmpty() && !lineEditCO->text().isEmpty()) {
        new QListWidgetItem( lineEditCS->text() + " " + comboBoxCP->currentText() + " " + object,
                             listWidgetC );
    }
}
    

void Konduit::VisualQueryBuilder::buttonRemoveO_clicked()
{
    listWidgetO->takeItem( listWidgetO->currentRow() );
}

void Konduit::VisualQueryBuilder::buttonRemoveC_clicked()
{
    listWidgetC->takeItem( listWidgetC->currentRow() );
}


void Konduit::VisualQueryBuilder::insertValueC( QString value )
{
    lineEditCO->setText( value );
}
    

void Konduit::VisualQueryBuilder::insertValueO( QString value )
{
    lineEditOO->setText( value );
}


void Konduit::VisualQueryBuilder::setCompletersCS( QStringList compList )
{
    //kDebug() << " " << compList;
    QCompleter* qComp = new QCompleter( compList + getVars() );//add all used variables to autocompletion
    qComp->setCompletionMode( QCompleter::PopupCompletion );
    
    //set completion list
    lineEditCS->setCompleter( qComp );
}

void Konduit::VisualQueryBuilder::setCompletersCP( QStringList compList )
{
    Q_UNUSED( compList );
    comboBoxCP->setExtraCompletion( getVars() );
}


void Konduit::VisualQueryBuilder::setCompletersCO( QStringList compList )
{
    //kDebug() << "WOO HOO!! " << compList;
    QCompleter* qComp = new QCompleter( compList + getVars() );//add all used variables to autocompletion
    qComp->setCompletionMode( QCompleter::PopupCompletion );
    
    //set completion list
    lineEditCO->setCompleter( qComp );
    //set type to URI
    //comboBoxTypesC->setCurrentIndex( comboBoxTypesO->findData( "URI" ) );
}

void Konduit::VisualQueryBuilder::setCompletersOS( QStringList compList )
{
    //kDebug() << " " << compList;
    QCompleter* qComp = new QCompleter( compList + getVars() );//add all used variables to autocompletion
    qComp->setCompletionMode( QCompleter::PopupCompletion );

    //set completion list
    lineEditOS->setCompleter( qComp );
}

void Konduit::VisualQueryBuilder::setCompletersOP( QStringList compList )
{
    Q_UNUSED( compList );
    comboBoxOP->setExtraCompletion( getVars() );
}


void Konduit::VisualQueryBuilder::setCompletersOO( QStringList compList )
{
    //kDebug() << "WOO HOO!! " << compList;
    QCompleter* qComp = new QCompleter( compList + getVars() );//add all used variables to autocompletion
    qComp->setCompletionMode( QCompleter::PopupCompletion );
    
    //set completion list
    lineEditOO->setCompleter( qComp );
    //set type to URI
    //comboBoxTypesC->setCurrentIndex( comboBoxTypesO->findData( "URI" ) );
}


/** **************** PRIVATE SLOTS *************************
*/


void Konduit::VisualQueryBuilder::languageChange()
{
    retranslateUi();
}


void Konduit::VisualQueryBuilder::textChangedCO()
{
    QString text = lineEditCO->text();

    if(text == lastObjectC) {//if unchaged, don't process again
        return;
    }
    lastObjectC = text;
    
    typesC.clear();
    typesC << VisualQueryBuilderConsts::xmlSchemaTypes();
    //kDebug() << "Adding all types: " << allTypes;
    if(text.isEmpty()) {
        refreshTypes();
        return;
    }

    QRegExp rx;
    rx.setPattern(VisualQueryBuilderConsts::decimalRegExp());
    if ( !rx.exactMatch( text ) ) { //indexIn( text ) < 0 ) {
        //kDebug() << "Removing: decimal";
        typesC.removeAt( typesC.indexOf( "decimal" ) );    
    }
    rx.setPattern(VisualQueryBuilderConsts::stringRegExp());
    if ( !rx.exactMatch( text ) ) { //indexIn( text ) < 0 ) {
        //kDebug() << "Removing: string";
        typesC.removeAt( typesC.indexOf( "string" ) );    
    }
    rx.setPattern(VisualQueryBuilderConsts::varRegExp());
    if ( !rx.exactMatch( text ) ) { //indexIn( text ) < 0 ) {
        //kDebug() << "Removing: var";
        typesC.removeAt( typesC.indexOf( "var" ) );    
    }
    rx.setPattern(VisualQueryBuilderConsts::uriRegExp());
    if ( !rx.exactMatch( text ) ) { //indexIn( text ) < 0 ) {
        //kDebug() << "Removing: URI";
        typesC.removeAt( typesC.indexOf( "URI" ) );    
    }
    rx.setPattern(VisualQueryBuilderConsts::dateTimeRegExp());
    if ( !rx.exactMatch( text ) ) { //indexIn( text ) < 0 ) {
        //kDebug() << "Removing: dateTime";
        typesC.removeAt( typesC.indexOf( "dateTime" ) );    
    }
    
    refreshTypes();
}


void Konduit::VisualQueryBuilder::textChangedOO()
{
    QString text = lineEditOO->text();
    
    if(text == lastObjectO) {//if unchaged, don't process again
        return;
    }
    lastObjectO = text;
    
    typesO.clear();
    typesO << VisualQueryBuilderConsts::xmlSchemaTypes();
    //kDebug() << "Adding all types: " << allTypes;
    if(text.isEmpty()) {
        refreshTypes();
        return;
    } 

    QRegExp rx;
    rx.setPattern(VisualQueryBuilderConsts::decimalRegExp());
    if ( !rx.exactMatch( text ) ) { //indexIn( text ) < 0 ) {
        //kDebug() << "Removing: decimal";
        typesO.removeAt( typesO.indexOf( "decimal" ) );    
    }
    rx.setPattern(VisualQueryBuilderConsts::stringRegExp());
    if ( !rx.exactMatch( text ) ) { //indexIn( text ) < 0 ) {
        //kDebug() << "Removing: string";
        typesO.removeAt( typesO.indexOf( "string" ) );    
    }
    rx.setPattern(VisualQueryBuilderConsts::varRegExp());
    if ( !rx.exactMatch( text ) ) { //indexIn( text ) < 0 ) {
        //kDebug() << "Removing: var";
        typesO.removeAt( typesO.indexOf( "var" ) );    
    }
    rx.setPattern(VisualQueryBuilderConsts::uriRegExp());
    if ( !rx.exactMatch( text ) ) { //indexIn( text ) < 0 ) {
        //kDebug() << "Removing: URI";
        typesO.removeAt( typesO.indexOf( "URI" ) );    
    }
    rx.setPattern(VisualQueryBuilderConsts::dateTimeRegExp());
    if ( !rx.exactMatch( text ) ) { //indexIn( text ) < 0 ) {
        //kDebug() << "Removing: dateTime";
        typesO.removeAt( typesO.indexOf( "dateTime" ) );    
    }
    
    refreshTypes();
}


void Konduit::VisualQueryBuilder::refreshTypes()
{
    //kDebug() << "Refreshing types";
    //kDebug() << "Refreshing types" << typesC << typesO;
    comboBoxTypesC->clear();
    comboBoxTypesC->insertItems( -1, typesC );
    comboBoxTypesC->setCurrentIndex( 0 );
    comboBoxTypesO->clear();
    comboBoxTypesO->insertItems( -1, typesO );
    comboBoxTypesO->setCurrentIndex( 0 );
}


void Konduit::VisualQueryBuilder::editingFinishedO()
{
    QString query = lineEditOS->text() + " " + comboBoxOP->currentText() + " " + VisualQueryBuilderConsts::getCanonicalForm( lineEditOO->text(), comboBoxTypesO->currentText() );
    query = "SELECT * WHERE { " + query + " } LIMIT 1";
    
    if( query == lastQueryO ) { //don't run same query many times
        return;
    }// else 
    lastQueryO = query;
    
    if( qTh->countQueryResults(query) == 0 ) {
        setColor( lineEditOS, comboBoxOP->lineEdit(), lineEditOO, comboBoxTypesO->currentText(), Qt::red );
    }
    else {
        setColor( lineEditOS, comboBoxOP->lineEdit(), lineEditOO, comboBoxTypesO->currentText(), Qt::white );
    }
    
    //generate random name
    srand( QTime::currentTime().msec() );
    QString varName;
    varName.setNum( 100 + rand() % 1000 );
    varName.prepend("xyz" + varName);

    //query for SUBJECT autocompletion
    query = "?" + varName + " " + comboBoxOP->currentText() + " " + VisualQueryBuilderConsts::getCanonicalForm( lineEditOO->text(), comboBoxTypesO->currentText() );
    setCompletersOS( qTh->runQuery( query, varName ) );
       
    //query for OBJECT
    query = lineEditOS->text() + " " + comboBoxOP->currentText() + " " + "?" + varName;
    setCompletersOO( qTh->runQuery( query, varName ) );

    //query for PREDICATE
    setCompletersOP( QStringList() );
}


void Konduit::VisualQueryBuilder::editingFinishedC()
{
    //bool popupCEnabled = comboBoxCP->view()->isVisible();
    QString query = lineEditCS->text() + " " + comboBoxCP->currentText() + " " + VisualQueryBuilderConsts::getCanonicalForm( lineEditCO->text(), comboBoxTypesC->currentText() );
    query = "SELECT * WHERE { " + query + " } LIMIT 1";
    
    if( query == lastQueryC ) { //don't run same query many times
        return;
    }// else 
    lastQueryC = query;

    if( qTh->countQueryResults(query) == 0 ) {
        setColor( lineEditCS, comboBoxCP->lineEdit(), lineEditCO, comboBoxTypesC->currentText(), Qt::red );
    }
    else {
        setColor( lineEditCS, comboBoxCP->lineEdit(), lineEditCO, comboBoxTypesC->currentText(),  Qt::white );
    }
    
    //generate random name
    srand( QTime::currentTime().msec() );
    QString varName;
    varName.setNum( 100 + rand() % 1000 );
    varName.prepend("xyz" + varName);

    //query for SUBJECT autocompletion
    query = "?" + varName + " " + comboBoxCP->currentText() + " " + VisualQueryBuilderConsts::getCanonicalForm( lineEditCO->text(), comboBoxTypesC->currentText() );
    setCompletersCS( qTh->runQuery( query, varName ) );
    
    //dispatch query to thread
    /*
    QueryThread* queryThread = new QueryThread;
    queryThread->setQuery( query );
    qRegisterMetaType<QStringList>("QStringList");
    connect( queryThread,  SIGNAL( queryDone( QStringList ) ), 
             this, SLOT( setCompletersCS( QStringList ) ) );
    qTh->start();
    */
    
      
    //query for OBJECT;
    query = lineEditCS->text() + " " + comboBoxCP->currentText() + " " + "?" + varName;
    setCompletersCO( qTh->runQuery( query, varName ) );
    //FIXME(?): set completer of lineEditCO in prefixed mode, and set type combo box

    //query for PREDICATE
    //query = lineEditCS->text() + " " + comboBoxCP->currentText() + " " + varName;
    //setCompletersCO( qTh->runQuery( query ) );
    setCompletersCP( QStringList() );

}


void Konduit::VisualQueryBuilder::varsChanged()
{
    QStringList result;
    QRegExp rx;
    rx.setPattern( VisualQueryBuilderConsts::varRegExp() );
    int pos;
    QString tempRes;
    
    //process the output part
    for(int i=0; i<listWidgetO->count(); i++) {
        QListWidgetItem *s = listWidgetO->item(i);
        
        pos = 0;
        QString str = s->text();
        while ((pos = rx.indexIn( str, pos )) != -1) {
            tempRes = rx.cap(1);
            if(!result.contains(tempRes) ) {
                result << tempRes;//rx.cap(1);
            }
            pos += rx.matchedLength();
        }
    }
    //process the conditional part
    for(int i=0; i<listWidgetC->count(); i++) {
        QListWidgetItem *s = listWidgetC->item(i);

        pos = 0;
        QString str = s->text();
        while ((pos = rx.indexIn( str, pos )) != -1) {
            tempRes = rx.cap(1);
            if(!result.contains(tempRes) ) {
                result << tempRes;//rx.cap(1);
            }
            pos += rx.matchedLength();
        }
    }
    
    //kDebug() << "......... " << result;
    varList.clear();
    varList << result;
}

/** **************** PRIVATE FUNCTIONS *************************
*/


QString Konduit::VisualQueryBuilder::constructQuery()
{
    QString query;
    
    //build the output part
    query += "CONSTRUCT { ";
    for(int i=0; i<listWidgetO->count(); i++) {
        QListWidgetItem *s = listWidgetO->item(i);
        query += s->text() + " . ";
    }
    //build the conditional part    
    query += " } WHERE { ";
    for(int i=0; i<listWidgetC->count(); i++) {
        QListWidgetItem *s = listWidgetC->item(i);
        query += s->text() + " . ";
    }
    query += " } ";
    
    //add the prefixes
    query = VisualQueryBuilderConsts::addPrefixes( query );
    
    return query;
}

    
void Konduit::VisualQueryBuilder::setColor( QLineEdit *lineEditS, QLineEdit *lineEditP, QLineEdit *lineEditO, QString objectType, QColor color )
{ 
    //kDebug() << "Recoloring with " << color;

    QRegExp rx;
    rx.setPattern( VisualQueryBuilderConsts::varRegExp() );
    
    if ( !rx.exactMatch( lineEditO->text() ) || objectType != "var" ) { //color all non-variables with given color (red or white)
        QPalette palette( lineEditO->palette() );
        palette.setColor( QPalette::Base, color );
        lineEditO->setPalette(palette);
    } else { //color variables with white
        QPalette palette( lineEditO->palette() );
        palette.setColor( QPalette::Base, Qt::white );
        lineEditO->setPalette(palette);
    }
    
    if ( !rx.exactMatch( lineEditP->text() ) ) { 
        QPalette palette( lineEditP->palette() );
        if( palette.color( QPalette::Base ) != color ) {//don't recolor with same color
            palette.setColor( QPalette::Base, color );
            lineEditP->setPalette(palette);
    }
    } else { //variables: always color with white
        QPalette palette( lineEditP->palette() );
        if( palette.color( QPalette::Base ) != color ) { //don't recolor with same color
            palette.setColor( QPalette::Base, Qt::white );
            lineEditP->setPalette(palette);
    }
    }
    if ( !rx.exactMatch( lineEditS->text() ) ) { 
        QPalette palette( lineEditCS->palette() );
        palette.setColor( QPalette::Base, color );
        lineEditS->setPalette(palette);
    } else { //variables: always color with white
        QPalette palette( lineEditP->palette() );
        palette.setColor( QPalette::Base, Qt::white );
        lineEditS->setPalette(palette);
    }
    
}

QStringList Konduit::VisualQueryBuilder::getVars()
{
    return varList;
}

#include "visualquerybuilder.moc"
