/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */
 
#include "visualquerybuilderconsts.h"

QMap<QString, QString> Konduit::VisualQueryBuilderConsts::prfxList;

    QString Konduit::VisualQueryBuilderConsts::decimalRegExp()//regular expression for decimal
    {
        return QString("([+-]?\\d+)(\\.\\d+)?");
    }
    QString  Konduit::VisualQueryBuilderConsts::stringRegExp()//string
    {
         return QString("[\\S\\s]*");
    }
    QString  Konduit::VisualQueryBuilderConsts::varRegExp()//var
    { 
        return QString("([\\?\\$]\\w+)");
    }
    QString  Konduit::VisualQueryBuilderConsts::uriRegExp()
    {
         return QString("((\"[\\S\\s]*\"\\^\\^)?<[\\S\\s]*>)|(\\S+:\\S+)");//"[<[\\w+://]?\\S*>]|[w+:w+]");//URI
    }
    QString  Konduit::VisualQueryBuilderConsts::dateTimeRegExp()//dateTime
    {
         return QString("(0[1-9]|[12][0-9]|3[01])[/]((0[1-9])|(1[012]))/\\d\\d\\d\\d");
    }
    
    QString Konduit::VisualQueryBuilderConsts::tripleRegExp() //regexp of a SPARQL triple
    {
        return QString("(\\S+[ \\.]*\\S+[ \\.]*\\S+)([ \\.]*)");
    }
    
    QString Konduit::VisualQueryBuilderConsts::queryRegExp() //regexp of a SPARQL query
    {
        return QString("^(.*)(CONSTRUCT\\s*\\{\\s*)((\\S*\\s*\\S*\\s*\\S*\\s*\\.?)+)(\\s*\\}\\s*)(WHERE\\s*\\{\\s*)((\\S*\\s*\\S*\\s*\\S*\\s*\\.?)+)(\\s*\\}\\s*)$");
    }
    
    QList<QString>  Konduit::VisualQueryBuilderConsts::xmlSchemaTypes() 
    {
        QList<QString> types;
        types << "URI" << "var" << "string" << "decimal" << "dateTime";
        return types;
    }
    
     
    QString Konduit::VisualQueryBuilderConsts::addPrefixes( QString query )
    {
        foreach( QString p, prefixes().keys() ) {
            if( query.contains( p + ":" ) ) {
                QString prefixNew = "PREFIX " + p + ": <" + prefixes().value( p ) + ">" ;
                if( !query.contains( prefixNew ) ) {
                    query.prepend( prefixNew + "\n" );
                }
                //query.prepend( "PREFIX " + p + ": <" + prefixes().value( p ) + ">\n" );
            }
        }
            return query;
    }
    
    
    QString  Konduit::VisualQueryBuilderConsts::getPrefixForm( QString predicate )
    {
        QString prefPred = predicate;
        bool prefixFound;
        
        foreach( QString p, prefixes().values() ) {
            prefixFound = false;
            if( prefPred.contains( p ) ) {
                //replace predicate with its (first and only) prefix
                prefPred.replace( p, prefixes().keys( p ).first() + ":" );
                prefixFound = true;
                break;
            }
        }
        //if no prefix found, surround predicate with < and >
        if( !prefixFound ) { 
            if(!prefPred.startsWith("<")) {
                prefPred.prepend("<");
            }
            if(!prefPred.endsWith(">")) {
                prefPred.append(">");
            }
        }
        return prefPred;
    }


    QString  Konduit::VisualQueryBuilderConsts::getCanonicalForm( QString res, QString type )
    { 
    //FIXME: complete list of types

    QString canonicalForm;
    
    if( type == "URI" ) {
        canonicalForm = res;
    }
    else if ( type == "var" ) {
        canonicalForm = res;
    }
    else if ( type == "string" ) {
        canonicalForm = "\"" + res + "\"^^<http://www.w3.org/2001/XMLSchema#string>" ;
    }
    else if ( type == "int" ) {
        canonicalForm = "\"" + res + "\"^^<http://www.w3.org/2001/XMLSchema#int>" ;
    }
    else if ( type == "dateTime" ) {
        canonicalForm = "\"" + res + "\"^^<http://www.w3.org/2001/XMLSchema#dateTime>" ;
    }
    else if ( type == "decimal" ) {
        canonicalForm = "\"" + res + "\"^^<http://www.w3.org/2001/XMLSchema#decimal>" ;
    }
    
    return canonicalForm; 
    }
    
    
    QMap<QString, QString>  Konduit::VisualQueryBuilderConsts::prefixes() 
    { 
        if ( prfxList.empty() ) {
        //build prefix list
        prfxList.insert("rdf", "http://www.w3.org/1999/02/22-rdf-syntax-ns#");
        prfxList.insert("rdfs", "http://www.w3.org/2000/01/rdf-schema#");
        prfxList.insert("sioc", "http://rdfs.org/sioc/ns#");
        prfxList.insert("rss", "http://purl.org/rss/1.0/");
        prfxList.insert("dc", "http://purl.org/dc/elements/1.1/");
        prfxList.insert("dcterms", "http://purl.org/dc/terms/");
        prfxList.insert("foaf", "http://xmlns.com/foaf/0.1/");
        prfxList.insert("nrl", "http://www.semanticdesktop.org/ontologies/2007/08/15/nrl#");
        prfxList.insert("xesam", "http://freedesktop.org/standards/xesam/1.0/core#");
        prfxList.insert("nao", "http://www.semanticdesktop.org/ontologies/2007/08/15/nao#");
        prfxList.insert("protege", "http://protege.stanford.edu/system#" );
        }

        return prfxList; 
    }


#include "visualquerybuilderconsts.moc"
