/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2008 Oszkar Ambrus <aoszkar@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef VISUALQUERYBUILDERCONSTS_H
#define VISUALQUERYBUILDERCONSTS_H

#include <KDebug>

namespace Konduit {

/*! Utility class for common constants and operations on queries
*/

class VisualQueryBuilderConsts
{

public:
    static QString decimalRegExp();
    static QString stringRegExp();
    static QString varRegExp();
    static QString uriRegExp();
    static QString dateTimeRegExp();
    static QString tripleRegExp();
    static QString queryRegExp();
    
    static QList<QString> xmlSchemaTypes() ;
    
    /** Returns the given query using PREFIXes
    */
    static QString addPrefixes( QString query );
    
    /** Returns the prefixed form of the given predicate
    */
   static  QString getPrefixForm( QString predicate );

    /** Returns the canonical URI form of the given resource type
    */
    static QString getCanonicalForm( QString res, QString type );
    
    
    static QMap<QString, QString> prefixes() ;
    
private: 
    static QMap<QString, QString> prfxList;

};



}

#endif

//#include "visualquerybuilderconsts.moc"

 /*

#ifndef VISUALQUERYBUILDERCONSTS_H
#define VISUALQUERYBUILDERCONSTS_H

#include <KDebug>

namespace Konduit {



class VisualQueryBuilderConsts
{

public:
    static QString decimalRegExp()//regular expression for decimal
    {
        return QString("([+-]?\\d+)(\\.\\d+)?");
    }
    static QString stringRegExp()//string
    {
         return QString("\\S*");
    }
    static QString varRegExp()//var
    { 
        return QString("([\\?\\$]\\w+)");
    }
    static QString uriRegExp()
    {
         return QString("<[\\w+://]?\\S*");//URI
    }
    static QString dateTimeRegExp()//dateTime
    {
         return QString("(0[1-9]|[12][0-9]|3[01])[/]((0[1-9])|(1[012]))/\\d\\d\\d\\d");
    }
    //static QMap<QString, QString> prefixesSparql() { return prefixes };
    
    static QList<QString> xmlSchemaTypes() 
    {
        QList<QString> types;
        types << "URI" << "var" << "string" << "decimal" << "dateTime";
        return types;
    }
    
    

   static  QString getPrefixForm( QString predicate )
    {
        QString prefPred = predicate;
        
        foreach( QString p, prefixes().values() ) {
            if( prefPred.contains( p ) ) {
                //replace predicate with its (first and only) prefix
                prefPred.replace( p, prefixes().keys( p ).first() + ":" );
                break;
            }
        }

        return prefPred;
    }


    static QString getCanonicalForm( QString res, QString type )
    { 
    //FIXME: complete list of types

    QString canonicalForm;
    
    if( type == "URI" ) {
        canonicalForm = res;
    }
    else if ( type == "var" ) {
        canonicalForm = res;
    }
    else if ( type == "string" ) {
        canonicalForm = "\"" + res + "\"^^<http://www.w3.org/2001/XMLSchema#string>" ;
    }
    else if ( type == "int" ) {
        canonicalForm = "\"" + res + "\"^^<http://www.w3.org/2001/XMLSchema#int>" ;
    }
    else if ( type == "dateTime" ) {
        canonicalForm = "\"" + res + "\"^^<http://www.w3.org/2001/XMLSchema#dateTime>" ;
    }
    else if ( type == "decimal" ) {
        canonicalForm = "\"" + res + "\"^^<http://www.w3.org/2001/XMLSchema#decimal>" ;
    }
    
    return canonicalForm; 
    }
    
    
    static QMap<QString, QString> prefixes() 
    { 
        if ( prfxList.empty() ) {
        //kDebug() << "Initializing prfxList";
        //build prfxList list
        prfxList.insert("rdf", "http://www.w3.org/1999/02/22-rdf-syntax-ns#");
        prfxList.insert("rdfs", "http://www.w3.org/2000/01/rdf-schema#");
        prfxList.insert("sioc", "http://rdfs.org/sioc/ns#");
        prfxList.insert("rss", "http://purl.org/rss/1.0/");
        prfxList.insert("dc", "http://purl.org/dc/elements/1.1/");
        prfxList.insert("dcterms", "http://purl.org/dc/terms/");
        prfxList.insert("foaf", "http://xmlns.com/foaf/0.1/");
        prfxList.insert("nrl", "http://www.semanticdesktop.org/ontologies/2007/08/15/nrl#");
        prfxList.insert("xesam", "http://freedesktop.org/standards/xesam/1.0/core#");
        prfxList.insert("nao", "http://www.semanticdesktop.org/ontologies/2007/08/15/nao#");
        prfxList.insert("protege", "http://protege.stanford.edu/system#" );
        }
        //kDebug() << "Returning prfxList" << prfxList;
        return prfxList; 
    }
    
private: 
        static QMap<QString, QString> prfxList;


};

QMap<QString, QString> VisualQueryBuilderConsts::prfxList;

}

#endif

//#include "visualquerybuilderconsts.moc"
*/
