/*
 * This file is part of Soprano Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _SOPRANO_VOCABULARY_H_
#define _SOPRANO_VOCABULARY_H_

#include <QtCore/QUrl>

namespace Konduit {
    namespace Vocabulary {
        /**
         * http://www.semanticdesktop.org/ontologies/2008/01/11/konduit#
         */
        QUrl vocabularyNamespace();

        /**
         * http://www.semanticdesktop.org/ontologies/2008/01/11/konduit#ConfigurationSetting 
         */
        QUrl ConfigurationSetting();

        /**
         * http://www.semanticdesktop.org/ontologies/2008/01/11/konduit#Element 
         */
        QUrl Element();

        /**
         * http://www.semanticdesktop.org/ontologies/2008/01/11/konduit#ElementPart 
         * 
         * Parent class for all parts of an element which are required, 
         * such as plugs or sockets. 
         */
        QUrl ElementPart();

        /**
         * http://www.semanticdesktop.org/ontologies/2008/01/11/konduit#Konduit 
         */
        QUrl Konduit();

    /**
         * http://www.semanticdesktop.org/ontologies/2008/01/11/konduit#KonduitBlackBox 
         */
        QUrl KonduitBlackBox();

        /**
         * http://www.semanticdesktop.org/ontologies/2008/01/11/konduit#Plug 
         */
        QUrl Plug();

        /**
         * http://www.semanticdesktop.org/ontologies/2008/01/11/konduit#Socket 
         */
        QUrl Socket();

        /**
         * http://www.semanticdesktop.org/ontologies/2008/01/11/konduit#configurationValue 
         */
        QUrl configurationValue();

        /**
         * http://www.semanticdesktop.org/ontologies/2008/01/11/konduit#elementType 
         * 
         * The Konduit plugin type, ie. the Plasma applet name. 
         */
        QUrl elementType();

        /**
         * http://www.semanticdesktop.org/ontologies/2008/01/11/konduit#parentElement 
         * 
         * The element this part belongs to. 
         */
        QUrl parentElement();

        /**
         * http://www.semanticdesktop.org/ontologies/2008/01/11/konduit#pluggedIn 
         * 
         * The socket a plug has been plugged into. 
         */
        QUrl pluggedIn();

        /**
         * http://www.semanticdesktop.org/ontologies/2008/01/11/konduit#geometry 
         * 
         * The geometry of an element.
         */
        QUrl geometry();
    }
}

#endif
