/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "datadumptest.h"
#include "../src/sparqlsourceelement.h"
#include "../src/datadumpelement.h"
#include "../src/plug.h"

#include <QtTest/QtTest>
#include <QtCore/QUrl>
#include <QtCore/QUuid>

#include <Soprano/Soprano>


using namespace Soprano;

static QUrl createRandomUri()
{
    // FIXME: check if the uri already exists
    QString uid = QUuid::createUuid().toString();
    uid = uid.mid( 1, uid.length()-2 );
    return QUrl( "http://soprano.org/test#" + uid );
}


static QList<Statement> createTestData( const Statement& s, int num )
{
    QList<Statement> sl;
    for( int i = 0; i < num; ++i ) {
        sl.append( Statement( s.subject().isEmpty() ? Node( createRandomUri() ) : s.subject(),
                              s.predicate().isEmpty() ? Node( createRandomUri() ) : s.predicate(),
                              s.object().isEmpty() ? Node( createRandomUri() ) : s.object(),
                              s.context() ) );
    }
    return sl;
}

void DataDumpTest::initTestCase()
{
    m_model = Soprano::createModel();
    Konduit::Element::setTestModel( m_model );

    // create some test data
    m_model->addStatements( createTestData( Statement(), 5 ) );
}


void DataDumpTest::testDump()
{
    Konduit::Elements::SimpleFileInputElement();

    Konduit::Elements::SparqlSourceElement* source = new Konduit::Elements::SparqlSourceElement( "construct { ?r ?p ?o } where "
                                                                             "{ ?r ?p ?o . }" );
    Konduit::Elements::DataDumpElement* sink = new Konduit::Elements::DataDumpElement();

    sink->plug()->connect( source->socket() );

    QVERIFY( sink->start() );

    delete sink;
    delete source;
}


void DataDumpTest::cleanupTestCase()
{
    delete m_model;
}

QTEST_MAIN( DataDumpTest )

#include "datadumptest.moc"
