/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "duplexertest.h"
#include "testsink.h"
#include "../src/simpledatasource.h"
#include "../src/dataduplexer.h"
#include "../src/plug.h"

#include <QtTest/QtTest>
#include <QtCore/QUrl>
#include <QtCore/QUuid>

#include <qtest_kde.h>

#include <Soprano/Soprano>


using namespace Soprano;

static QUrl createRandomUri()
{
    // FIXME: check if the uri already exists
    QString uid = QUuid::createUuid().toString();
    uid = uid.mid( 1, uid.length()-2 );
    return QUrl( "http://soprano.org/test#" + uid );
}


static QList<Statement> createTestData( const Statement& s, int num )
{
    QList<Statement> sl;
    for( int i = 0; i < num; ++i ) {
        sl.append( Statement( s.subject().isEmpty() ? Node( createRandomUri() ) : s.subject(),
                              s.predicate().isEmpty() ? Node( createRandomUri() ) : s.predicate(),
                              s.object().isEmpty() ? Node( createRandomUri() ) : s.object(),
                              s.context() ) );
    }
    return sl;
}

void DuplexerTest::initTestCase()
{
}


void DuplexerTest::testSingleDuplexer()
{
    Konduit::Elements::SimpleDataSource* source = new Konduit::Elements::SimpleDataSource( 0, QString(), 0, createTestData( Statement(), 5 ) );
    Konduit::Elements::DataDuplexer* duplexer = new Konduit::Elements::DataDuplexer( 0, QVariantList() );
    Konduit::TestSink* sink1 = new Konduit::TestSink( source->data() );
    Konduit::TestSink* sink2 = new Konduit::TestSink( source->data() );

    duplexer->plug()->plugIn( source->socket() );
    sink1->plug()->plugIn( duplexer->firstSocket( ) );
    sink2->plug()->plugIn( duplexer->secondSocket( ) );

    QVERIFY( sink1->start() );
    QVERIFY( sink2->start() );

    delete sink1;
    delete sink2;
    delete duplexer;
    delete source;
}


void DuplexerTest::testChainingDuplexer()
{
    Konduit::Elements::SimpleDataSource* source = new Konduit::Elements::SimpleDataSource( 0, QString(), 0, createTestData( Statement(), 5 ) );

    Konduit::Elements::DataDuplexer* duplexer1 = new Konduit::Elements::DataDuplexer( 0, QVariantList() );
    Konduit::Elements::DataDuplexer* duplexer2 = new Konduit::Elements::DataDuplexer( 0, QVariantList() );
    Konduit::Elements::DataDuplexer* duplexer3 = new Konduit::Elements::DataDuplexer( 0, QVariantList() );

    Konduit::TestSink* sink1 = new Konduit::TestSink( source->data() );
    Konduit::TestSink* sink2 = new Konduit::TestSink( source->data() );
    Konduit::TestSink* sink3 = new Konduit::TestSink( source->data() );
    Konduit::TestSink* sink4 = new Konduit::TestSink( source->data() );

    duplexer1->plug()->plugIn( source->socket() );
    duplexer2->plug()->plugIn( duplexer1->firstSocket() );
    duplexer3->plug()->plugIn( duplexer1->secondSocket() );

    sink1->plug()->plugIn( duplexer2->firstSocket( ) );
    sink2->plug()->plugIn( duplexer2->secondSocket( ) );
    sink3->plug()->plugIn( duplexer3->firstSocket( ) );
    sink4->plug()->plugIn( duplexer3->secondSocket( ) );

    QVERIFY( sink1->start() );
    QVERIFY( sink2->start() );
    QVERIFY( sink3->start() );
    QVERIFY( sink4->start() );

    delete sink1;
    delete sink2;
    delete sink3;
    delete sink4;
    delete duplexer1;
    delete duplexer2;
    delete duplexer3;
    delete source;
}


void DuplexerTest::cleanupTestCase()
{
}

QTEST_KDEMAIN( DuplexerTest, NoGUI )

#include "duplexertest.moc"
