/*
 * This file is part of Konduit Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "testsink.h"
#include "plug.h"
#include "source.h"
#include "socket.h"

#include <Soprano/StatementIterator>


Konduit::TestSink::TestSink( const QList<Soprano::Statement>& statements )
    : SinkElement( 0, QString(), 0 ),
      m_statements( statements )
{
    addPlug( "input" );
}


Konduit::TestSink::~TestSink()
{
}


bool Konduit::TestSink::start()
{
    if ( !plug()->socket() ) {
        return false;
    }

    QList<Soprano::Statement> sl = plug()->socket()->data().allStatements();
    if ( sl.count() != m_statements.count() ) {
        return false;
    }
    Q_FOREACH( Soprano::Statement s, sl ) {
        if ( !m_statements.contains( s ) ) {
            return false;
        }
    }
    return true;
}
